@@ -13,31 +13,6 @@ unsigned G_SCROLL_BAR_AREA_COLOR;
 unsigned G_SCROLL_BAR_BAR_COLOR;
 unsigned G_SCROLL_BAR_FRAME_COLOR;
 
-/* prototypes */
-int is_in_str(unsigned char *, int);
-void select_menu(struct terminal *, struct menu *);
-void count_menu_size(struct terminal *, struct menu *);
-void scroll_menu(struct menu *, int);
-void display_menu_txt(struct terminal *, struct menu *);
-void display_menu_item_gfx(struct terminal *, struct menu *, int);
-void display_menu_gfx(struct terminal *, struct menu *);
-void display_mainmenu(struct terminal *, struct mainmenu *);
-void select_mainmenu(struct terminal *, struct mainmenu *);
-void u_display_dlg_item(struct terminal *, void *);
-void x_display_dlg_item(struct dialog_data *, struct dialog_item_data *, int);
-void dlg_select_item(struct dialog_data *, struct dialog_item_data *);
-void dlg_set_history(struct dialog_item_data *);
-int dlg_mouse(struct dialog_data *, struct dialog_item_data *, struct event *);
-void redraw_dialog_items(struct terminal *, struct dialog_data *);
-int dlg_is_braille_moving(struct dialog_data *);
-void redraw_dialog(struct terminal *, struct dialog_data *);
-void tab_compl(struct terminal *, unsigned char *, struct window *);
-void do_tab_compl(struct terminal *, struct list_head *, struct window *);
-int msg_box_button(struct dialog_data *, struct dialog_item_data *);
-int input_field_cancel(struct dialog_data *, struct dialog_item_data *);
-int input_field_ok(struct dialog_data *, struct dialog_item_data *);
-
-
 struct memory_list *getml(void *p, ...)
 {
 	struct memory_list *ml;
@@ -93,15 +68,41 @@ void freeml(struct memory_list *ml)
 	mem_free(ml);
 }
 
-#ifndef G
-#define txtlen (int)strlen
-#else
-#define txtlen(x) (F ? g_text_width(bfu_style_wb, x) : (int)strlen(x))
+static inline int is_utf_8(struct terminal *term)
+{
+#ifdef G
+	if (F) return 1;
 #endif
+#ifdef ENABLE_UTF8
+	if (term->spec->charset == utf8_table) return 1;
+#endif
+	return 0;
+}
+
+static inline int ttxtlen(struct terminal *term, unsigned char *s)
+{
+#ifdef ENABLE_UTF8
+	if (term->spec->charset == utf8_table)
+		return strlen_utf8(s);
+#endif
+	return strlen(s);
+}
+
+static inline int txtlen(struct terminal *term, unsigned char *s)
+{
+	if (!F) {
+		return ttxtlen(term, s);
+	}
+#ifdef G
+	else {
+		return g_text_width(bfu_style_wb, s);
+	}
+#endif
+}
 
 #ifdef G
 
-int is_in_str(unsigned char *str, int u)
+static int is_in_str(unsigned char *str, int u)
 {
 	while (*str) {
 		int v;
@@ -215,7 +216,7 @@ void do_menu(struct terminal *term, struct menu_item *items, void *data)
 	do_menu_selected(term, items, data, 0);
 }
 
-void select_menu(struct terminal *term, struct menu *menu)
+static void select_menu(struct terminal *term, struct menu *menu)
 {
 	struct menu_item *it = &menu->items[menu->selected];
 	void (*func)(struct terminal *, void *, void *) = it->func;
@@ -229,14 +230,14 @@ void select_menu(struct terminal *term, struct menu *menu)
 	func(term, data1, data2);
 }
 
-void count_menu_size(struct terminal *term, struct menu *menu)
+static void count_menu_size(struct terminal *term, struct menu *menu)
 {
 	int sx = term->x;
 	int sy = term->y;
 	int mx = gf_val(4, 2 * (G_MENU_LEFT_BORDER + G_MENU_LEFT_INNER_BORDER));
 	int my;
 	for (my = 0; menu->items[my].text; my++) {
-		int s = txtlen(_(menu->items[my].text, term)) + txtlen(_(menu->items[my].rtext, term)) + gf_val(MENU_HOTKEY_SPACE, G_MENU_HOTKEY_SPACE) * (_(menu->items[my].rtext, term)[0] != 0) + gf_val(4, 2 * (G_MENU_LEFT_BORDER + G_MENU_LEFT_INNER_BORDER));
+		int s = txtlen(term, _(menu->items[my].text, term)) + txtlen(term, _(menu->items[my].rtext, term)) + gf_val(MENU_HOTKEY_SPACE, G_MENU_HOTKEY_SPACE) * (_(menu->items[my].rtext, term)[0] != 0) + gf_val(4, 2 * (G_MENU_LEFT_BORDER + G_MENU_LEFT_INNER_BORDER));
 		if (s > mx) mx = s;
 	}
 	menu->ni = my;
@@ -270,7 +271,7 @@ void count_menu_size(struct terminal *term, struct menu *menu)
 #endif
 }
 
-void scroll_menu(struct menu *menu, int d)
+static void scroll_menu(struct menu *menu, int d)
 {
 	int c = 0;
 	int w = menu->nview;
@@ -295,17 +296,17 @@ void scroll_menu(struct menu *menu, int d)
 	if (menu->view < 0) menu->view = 0;
 }
 
-void display_menu_txt(struct terminal *term, struct menu *menu)
+static void display_menu_txt(struct terminal *term, struct menu *menu)
 {
 	int p, s;
 	int setc = 0;
-	fill_area(term, menu->x+1, menu->y+1, menu->xw-2, menu->yw-2, COLOR_MENU);
+	fill_area(term, menu->x+1, menu->y+1, menu->xw-2, menu->yw-2, ' ', COLOR_MENU);
 	draw_frame(term, menu->x, menu->y, menu->xw, menu->yw, COLOR_MENU_FRAME, 1);
 	set_window_ptr(menu->win, menu->x, menu->y);
 	for (p = menu->view, s = menu->y + 1; p < menu->ni && p < menu->view + menu->yw - 2; p++, s++) {
 		int x;
 		int h = 0;
-		unsigned char c;
+		unsigned c;
 		unsigned char *tmptext = _(menu->items[p].text, term);
 		int co = p == menu->selected ? h = 1, COLOR_MENU_SELECTED : COLOR_MENU;
 		if (h) {
@@ -313,27 +314,34 @@ void display_menu_txt(struct terminal *term, struct menu *menu)
 			set_cursor(term, menu->x + 1 + !!term->spec->braille, s, term->x - 1, term->y - 1);
 			/*set_window_ptr(menu->win, menu->x+3, s+1);*/
 			set_window_ptr(menu->win, menu->x+menu->xw, s);
-			fill_area(term, menu->x+1, s, menu->xw-2, 1, co);
+			fill_area(term, menu->x+1, s, menu->xw-2, 1, ' ', co);
 		}
 		if (term->spec->braille) h = 1;
 		if (menu->items[p].hotkey != M_BAR || (tmptext[0])) {
-			int l = strlen(_(menu->items[p].rtext, term));
 			unsigned char *rt = _(menu->items[p].rtext, term);
+			int l = ttxtlen(term, rt);
 			unsigned char *ht = _(menu->items[p].hotkey, term);
-			for (x = l - 1; x >= 0 && (term->spec->braille || menu->xw - 4 >= l - x) && (c = rt[x]); x--) {
-				if (!term->spec->braille)
-					set_char(term, menu->x + menu->xw - 2 - l + x, s, c | co);
-				else
-					set_char(term, menu->x + strlen(tmptext) + 4 + x + 2, s, c | COLOR_MENU_HOTKEY);
+			for (x = 0;; x++) {
+				c = GET_TERM_CHAR(term, &rt);
+				if (!c) break;
+				if (!term->spec->braille) {
+					if (menu->xw - 4 >= l - x)
+						set_char(term, menu->x + menu->xw - 2 - l + x, s, c, co);
+				} else {
+					set_char(term, menu->x + ttxtlen(term, tmptext) + 4 + x + 2, s, c, COLOR_MENU_HOTKEY);
+				}
+			}
+			for (x = 0; x < menu->xw - 4; x++) {
+				c = GET_TERM_CHAR(term, &tmptext);
+				if (!c) break;
+				set_char(term, menu->x + x + 2 + 2 * !!term->spec->braille, s, c, !h && cp_strchr(term->spec->charset, ht, uni_upcase(c)) ? h = 1, COLOR_MENU_HOTKEY : co);
 			}
-			for (x = 0; x < menu->xw - 4 && (c = tmptext[x]); x++)
-				set_char(term, menu->x + x + 2 + 2 * !!term->spec->braille, s, !h && strchr(ht, upcase(c)) ? h = 1, COLOR_MENU_HOTKEY | c : co | c);
 			if (term->spec->braille && *ht)
-				set_char(term, menu->x + 2, s, ht[0] | COLOR_MENU_HOTKEY);
+				set_char(term, menu->x + 2, s, ht[0], COLOR_MENU_HOTKEY);
 		} else {
-			set_char(term, menu->x, s, COLOR_MENU_FRAME | ATTR_FRAME | 0xc3);
-			fill_area(term, menu->x+1, s, menu->xw-2, 1, COLOR_MENU_FRAME | ATTR_FRAME | 0xc4);
-			set_char(term, menu->x+menu->xw-1, s, COLOR_MENU_FRAME | ATTR_FRAME | 0xb4);
+			set_char(term, menu->x, s, 0xc3, COLOR_MENU_FRAME | ATTR_FRAME);
+			fill_area(term, menu->x+1, s, menu->xw-2, 1, 0xc4, COLOR_MENU_FRAME | ATTR_FRAME);
+			set_char(term, menu->x+menu->xw-1, s, 0xb4, COLOR_MENU_FRAME | ATTR_FRAME);
 		}
 	}
 	if (!setc && term->spec->braille) {
@@ -348,7 +356,7 @@ int menu_oldsel = -1;
 
 int menu_ptr_set;
 
-void display_menu_item_gfx(struct terminal *term, struct menu *menu, int it)
+static void display_menu_item_gfx(struct terminal *term, struct menu *menu, int it)
 {
 	struct menu_item *item = &menu->items[it];
 	struct graphics_device *dev = term->dev;
@@ -409,7 +417,7 @@ void display_menu_item_gfx(struct terminal *term, struct menu *menu, int it)
 	}
 }
 
-void display_menu_gfx(struct terminal *term, struct menu *menu)
+static void display_menu_gfx(struct terminal *term, struct menu *menu)
 {
 	int p;
 	struct graphics_device *dev = term->dev;
@@ -543,10 +551,10 @@ void menu_func(struct window *win, struct event *ev, int fwd)
 				if ((menu->view += menu->yw / gf_val(1, G_BFU_FONT_SIZE) - 2) >= menu->ni - menu->yw + 2) menu->view = menu->ni - menu->yw + 2;
 				scroll_menu(menu, 1);
 			}
-			else if (ev->x > ' ' && ev->x < 256) {
+			else if (ev->x > ' ') {
 				int i;
 				for (i = 0; i < menu->ni; i++)
-					if (strchr(_(menu->items[i].hotkey, win->term), upcase(ev->x))) {
+					if (cp_strchr(win->term->spec->charset, _(menu->items[i].hotkey, win->term), uni_upcase(ev->x))) {
 						menu->selected = i;
 						scroll_menu(menu, 0);
 						s = 1;
@@ -601,33 +609,37 @@ void do_mainmenu(struct terminal *term, struct menu_item *items, void *data, int
 	}
 }
 
-void display_mainmenu(struct terminal *term, struct mainmenu *menu)
+static void display_mainmenu(struct terminal *term, struct mainmenu *menu)
 {
 	if (!F) {
 		int i;
 		int p = 2;
-		fill_area(term, 0, 0, term->x, 1, COLOR_MAINMENU | ' ');
+		fill_area(term, 0, 0, term->x, 1, ' ', COLOR_MAINMENU);
 		for (i = 0; menu->items[i].text; i++) {
 			int s = 0;
-			int j;
-			unsigned char c;
+			unsigned c;
 			unsigned char *tmptext = _(menu->items[i].text, term);
 			unsigned char *ht = _(menu->items[i].hotkey, term);
 			int co = i == menu->selected ? s = 1, COLOR_MAINMENU_SELECTED : COLOR_MAINMENU;
-			if (s) {
-				fill_area(term, p, 0, 2, 1, co);
-				fill_area(term, p+strlen(tmptext)+2, 0, 2, 1, co);
+			if (i == menu->selected) {
+				fill_area(term, p, 0, 2, 1, ' ', co);
 				menu->sp = p;
 				set_cursor(term, p, 0, term->x - 1, term->y - 1);
 				set_window_ptr(menu->win, p, 1);
 			}
 			if (term->spec->braille) {
 				s = 1;
-				if (*ht) set_char(term, p, 0, *ht | COLOR_MAINMENU_HOTKEY);
+				if (*ht) set_char(term, p, 0, *ht, COLOR_MAINMENU_HOTKEY);
 			}
 			p += 2;
-			for (j = 0; (c = tmptext[j]); j++, p++)
-				set_char(term, p, 0, (!s && strchr(ht, upcase(c)) ? s = 1, COLOR_MAINMENU_HOTKEY : co) | c);
+			for (;; p++) {
+				c = GET_TERM_CHAR(term, &tmptext);
+				if (!c) break;
+				set_char(term, p, 0, c, !s && cp_strchr(term->spec->charset, ht, uni_upcase(c)) ? s = 1, COLOR_MAINMENU_HOTKEY : co);
+			}
+			if (i == menu->selected) {
+				fill_area(term, p, 0, 2, 1, ' ', co);
+			}
 			p += 2;
 		}
 		menu->ni = i;
@@ -660,7 +672,7 @@ void display_mainmenu(struct terminal *term, struct mainmenu *menu)
 	}
 }
 
-void select_mainmenu(struct terminal *term, struct mainmenu *menu)
+static void select_mainmenu(struct terminal *term, struct mainmenu *menu)
 {
 	struct menu_item *it = &menu->items[menu->selected];
 	if (menu->selected < 0 || menu->selected >= menu->ni || it->hotkey == M_BAR) return;
@@ -694,7 +706,7 @@ void mainmenu_func(struct window *win, struct event *ev, int fwd)
 				for (i = 0; i < menu->ni; i++) {
 					int o = p;
 					unsigned char *tmptext = _(menu->items[i].text, win->term);
-					p += txtlen(tmptext) + gf_val(4, 2 * G_MAINMENU_BORDER);
+					p += txtlen(win->term, tmptext) + gf_val(4, 2 * G_MAINMENU_BORDER);
 					if (ev->x >= o && ev->x < p) {
 						menu->selected = i;
 						draw_to_window(win, (void (*)(struct terminal *, void *))display_mainmenu, menu);
@@ -726,7 +738,7 @@ void mainmenu_func(struct window *win, struct event *ev, int fwd)
 				int i;
 				s = 1;
 				for (i = 0; i < menu->ni; i++)
-					if (strchr(_(menu->items[i].hotkey, win->term), upcase(ev->x))) {
+					if (cp_strchr(win->term->spec->charset, _(menu->items[i].hotkey, win->term), uni_upcase(ev->x))) {
 						menu->selected = i;
 						s = 2;
 					}
@@ -795,7 +807,8 @@ void display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int
 	struct terminal *term = dlg->win->term;
 	if (!F) switch (di->item->type) {
 		int co;
-		unsigned char *text, *t, *tt;
+		unsigned char *text, *t;
+		int vposlen, cposlen;
 		case D_CHECKBOX:
 			if (di->item->gid)	/* radio */
 			{
@@ -814,18 +827,32 @@ void display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int
 			break;
 		case D_FIELD:
 		case D_FIELD_PASS:
-			if (di->vpos + di->l <= di->cpos) di->vpos = di->cpos - di->l + 1;
+			fill_area(term, di->x, di->y, di->l, 1, ' ', COLOR_DIALOG_FIELD);
 			if (di->vpos > di->cpos) di->vpos = di->cpos;
-			if (di->vpos < 0) di->vpos = 0;
-			fill_area(term, di->x, di->y, di->l, 1, COLOR_DIALOG_FIELD);
+			vposlen = ttxtlen(term, di->cdata + di->vpos);
+			cposlen = ttxtlen(term, di->cdata + di->cpos);
+			if (!di->l) {
+				di->vpos = di->cpos;
+				vposlen = cposlen;
+			} else {
+				while (vposlen - cposlen > di->l - 1) {
+					t = di->cdata + di->vpos;
+					GET_TERM_CHAR(term, &t);
+					di->vpos = t - di->cdata;
+					vposlen--;
+				}
+			}
 			if (di->item->type == D_FIELD_PASS) {
-				t = stracpy(di->cdata);
-				for (tt = t; *tt; *tt++ = '*');
-			} else t = di->cdata;
-			print_text(term, di->x, di->y, strlen(t + di->vpos) <= (size_t)di->l ? (int)strlen(t + di->vpos) : di->l, t + di->vpos, COLOR_DIALOG_FIELD_TEXT);
+				t = mem_alloc(vposlen + 1);
+				memset(t, '*', vposlen);
+				t[vposlen] = 0;
+			} else {
+				t = di->cdata + di->vpos;
+			}
+			print_text(term, di->x, di->y, di->l, t, COLOR_DIALOG_FIELD_TEXT);
 			if (di->item->type == D_FIELD_PASS) mem_free(t);
 			if (sel) {
-				set_cursor(term, di->x + di->cpos - di->vpos, di->y, di->x + di->cpos - di->vpos, di->y);
+				set_cursor(term, di->x + vposlen - cposlen, di->y, di->x + vposlen - cposlen, di->y);
 				set_window_ptr(dlg->win, di->x, di->y);
 			}
 			break;
@@ -833,8 +860,8 @@ void display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int
 			co = sel ? COLOR_DIALOG_BUTTON_SELECTED : COLOR_DIALOG_BUTTON;
 			text = _(di->item->text, term);
 			print_text(term, di->x, di->y, 2, "[ ", co);
-			print_text(term, di->x + 2, di->y, strlen(text), text, co);
-			print_text(term, di->x + 2 + strlen(text), di->y, 2, " ]", co);
+			print_text(term, di->x + 2, di->y, ttxtlen(term, text), text, co);
+			print_text(term, di->x + 2 + ttxtlen(term, text), di->y, 2, " ]", co);
 			if (sel) {
 				set_cursor(term, di->x + 2, di->y, di->x + 2, di->y);
 				set_window_ptr(dlg->win, di->x, di->y);
@@ -964,20 +991,20 @@ struct dspd {
 	int sel;
 };
 
-void u_display_dlg_item(struct terminal *term, void *p)
+static void u_display_dlg_item(struct terminal *term, void *p)
 {
 	struct dspd *d = p;
 	display_dlg_item(d->dlg, d->di, d->sel);
 }
 
-void x_display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int sel)
+static void x_display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int sel)
 {
 	struct dspd dspd;
 	dspd.dlg = dlg, dspd.di = di, dspd.sel = sel;
 	draw_to_window(dlg->win, u_display_dlg_item, &dspd);
 }
 
-void dlg_select_item(struct dialog_data *dlg, struct dialog_item_data *di)
+static void dlg_select_item(struct dialog_data *dlg, struct dialog_item_data *di)
 {
 	if (di->item->type == D_CHECKBOX) {
 		if (!di->item->gid) di -> checked = *(int *)di->cdata = !*(int *)di->cdata;
@@ -997,7 +1024,7 @@ void dlg_select_item(struct dialog_data *dlg, struct dialog_item_data *di)
 	else if (di->item->type == D_BUTTON) di->item->fn(dlg, di);
 }
 
-void dlg_set_history(struct dialog_item_data *di)
+static void dlg_set_history(struct dialog_item_data *di)
 {
 	unsigned char *s = "";
 	int l;
@@ -1009,11 +1036,11 @@ void dlg_set_history(struct dialog_item_data *di)
 	di->vpos = 0;
 }
 
-int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struct event *ev)
+static int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struct event *ev)
 {
 	switch (di->item->type) {
 		case D_BUTTON:
-			if (gf_val(ev->y != di->y, ev->y < di->y || ev->y >= di->y + G_BFU_FONT_SIZE) || ev->x < di->x || ev->x >= di->x + (int)gf_val(strlen(_(di->item->text, dlg->win->term)) + 4, (size_t)di->l)) return 0;
+			if (gf_val(ev->y != di->y, ev->y < di->y || ev->y >= di->y + G_BFU_FONT_SIZE) || ev->x < di->x || ev->x >= di->x + gf_val(ttxtlen(dlg->win->term, _(di->item->text, dlg->win->term)) + 4, di->l)) return 0;
 			if (dlg->selected != di - dlg->items) {
 				x_display_dlg_item(dlg, &dlg->items[dlg->selected], 0);
 				dlg->selected = di - dlg->items;
@@ -1024,9 +1051,8 @@ int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struct event
 		case D_FIELD:
 		case D_FIELD_PASS:
 			if (gf_val(ev->y != di->y, ev->y < di->y || ev->y >= di->y + G_BFU_FONT_SIZE) || ev->x < di->x || ev->x >= di->x + di->l) return 0;
-			if (!F) {
+			if (!is_utf_8(dlg->win->term)) {
 				if ((size_t)(di->cpos = di->vpos + ev->x - di->x) > strlen(di->cdata)) di->cpos = strlen(di->cdata);
-#ifdef G
 			} else {
 				int p, u;
 				unsigned char *t = di->cdata;
@@ -1036,10 +1062,12 @@ int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struct event
 					if (!*t) break;
 					GET_UTF_8(t, u);
 					if (!u) continue;
-					p += g_char_width(bfu_style_wb_mono, u);
+					if (!F) p++;
+#ifdef G
+					else p += g_char_width(bfu_style_wb_mono, u);
+#endif
 					if (p > ev->x) break;
 				}
-#endif
 			}
 			if (dlg->selected != di - dlg->items) {
 				x_display_dlg_item(dlg, &dlg->items[dlg->selected], 0);
@@ -1060,18 +1088,18 @@ int dlg_mouse(struct dialog_data *dlg, struct dialog_item_data *di, struct event
 	return 0;
 }
 
-void redraw_dialog_items(struct terminal *term, struct dialog_data *dlg)
+static void redraw_dialog_items(struct terminal *term, struct dialog_data *dlg)
 {
 	int i;
 	for (i = 0; i < dlg->n; i++) display_dlg_item(dlg, &dlg->items[i], i == dlg->selected);
 }
 
-int dlg_is_braille_moving(struct dialog_data *dlg)
+static int dlg_is_braille_moving(struct dialog_data *dlg)
 {
 	return dlg->win->term->spec->braille && (dlg->dlg->fn == msg_box_fn || dlg->dlg->fn == download_window_function);
 }
 
-void redraw_dialog(struct terminal *term, struct dialog_data *dlg)
+static void redraw_dialog(struct terminal *term, struct dialog_data *dlg)
 {
 #ifdef G
 	int i;
@@ -1093,7 +1121,7 @@ void redraw_dialog(struct terminal *term, struct dialog_data *dlg)
 #endif
 }
 
-void tab_compl(struct terminal *term, unsigned char *item, struct window *win)
+static void tab_compl(struct terminal *term, unsigned char *item, struct window *win)
 {
 	struct event ev = {EV_REDRAW, 0, 0, 0};
 	struct dialog_item_data *di = &((struct dialog_data*)win->data)->items[((struct dialog_data*)win->data)->selected];
@@ -1108,7 +1136,7 @@ void tab_compl(struct terminal *term, unsigned char *item, struct window *win)
 	dialog_func(win, &ev, 0);
 }
 
-void do_tab_compl(struct terminal *term, struct list_head *history, struct window *win)
+static void do_tab_compl(struct terminal *term, struct list_head *history, struct window *win)
 {
 	unsigned char *cdata = ((struct dialog_data*)win->data)->items[((struct dialog_data*)win->data)->selected].cdata;
 	int l = strlen(cdata), n = 0;
@@ -1169,7 +1197,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
 					} else if (*(int *)di->cdata) di->checked = 1;
 				} 
 				init_list(di->history);
-				di->cur_hist = (struct history_item *)&di->history;
+				di->cur_hist = (struct history_item *)(void *)&di->history;
 				if (di->item->type == D_FIELD || di->item->type == D_FIELD_PASS) {
 					if (di->item->history) {
 						struct history_item *j;
@@ -1235,28 +1263,24 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
 				}
 				if (ev->x == KBD_RIGHT) {
 					if ((size_t)di->cpos < strlen(di->cdata)) {
-						if (!F) di->cpos++;
-#ifdef G
+						if (!is_utf_8(term)) di->cpos++;
 						else {
 							int u;
 							unsigned char *p = di->cdata + di->cpos;
 							GET_UTF_8(p, u);
 							di->cpos = p - di->cdata;
 						}
-#endif
 					}
 					goto dsp_f;
 				}
 				if (ev->x == KBD_LEFT) {
 					if (di->cpos > 0) {
-						if (!F) di->cpos--;
-#ifdef G
+						if (!is_utf_8(term)) di->cpos--;
 						else {
 							unsigned char *p = di->cdata + di->cpos;
 							BACK_UTF_8(p, di->cdata);
 							di->cpos = p - di->cdata;
 						}
-#endif
 					}
 					goto dsp_f;
 				}
@@ -1271,12 +1295,11 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
 				if (ev->x >= ' ' && !(ev->y & (KBD_CTRL | KBD_ALT))) {
 					unsigned char *u;
 					unsigned char p[2] = { 0, 0 };
-					if (!F) p[0] = ev->x, u = p;
-#ifdef G
-					else {
+					if (!is_utf_8(term)) {
+						p[0] = ev->x, u = p;
+					} else {
 						u = encode_utf_8(ev->x);
 					}
-#endif
 					if (strlen(di->cdata) < di->item->dlen - strlen(u)) {
 						memmove(di->cdata + di->cpos + strlen(u), di->cdata + di->cpos, strlen(di->cdata) - di->cpos + 1);
 						memcpy(&di->cdata[di->cpos], u, strlen(u));
@@ -1287,8 +1310,7 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
 				if (ev->x == KBD_BS) {
 					if (di->cpos) {
 						int s = 1;
-#ifdef G
-						if (F) {
+						if (is_utf_8(term)) {
 							unsigned u;
 							unsigned char *p, *pp;
 							p = di->cdata;
@@ -1298,7 +1320,6 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
 							if (p < di->cdata + di->cpos) goto a;
 							s = p - pp;
 						}
-#endif
 						memmove(di->cdata + di->cpos - s, di->cdata + di->cpos, strlen(di->cdata) - di->cpos + s);
 						di->cpos -= s;
 					}
@@ -1307,20 +1328,18 @@ void dialog_func(struct window *win, struct event *ev, int fwd)
 				if (ev->x == KBD_DEL || (upcase(ev->x) == 'D' && ev->y & KBD_CTRL)) {
 					if ((size_t)di->cpos < strlen(di->cdata)) {
 						int s = 1;
-#ifdef G
-						if (F) {
+						if (is_utf_8(term)) {
 							unsigned u;
 							unsigned char *p = di->cdata + di->cpos;
 							GET_UTF_8(p, u);
 							s = p - (di->cdata + di->cpos);
 						}
-#endif
 						memmove(di->cdata + di->cpos, di->cdata + di->cpos + s, strlen(di->cdata) - di->cpos + s);
 					}
 					goto dsp_f;
 				}
 				if (upcase(ev->x) == 'U' && ev->y & KBD_CTRL) {
-					char *a = memacpy(di->cdata, di->cpos);
+					unsigned char *a = memacpy(di->cdata, di->cpos);
 					if (a) {
 						set_clipboard_text(term, a);
 						mem_free(a);
@@ -1376,8 +1395,9 @@ clipbd_paste:
 				break;
 			}
 			gh:
-			if (ev->x > ' ') for (i = 0; i < dlg->n; i++)
-				if (dlg->dlg->items[i].type == D_BUTTON && upcase(_(dlg->dlg->items[i].text, term)[0]) == upcase(ev->x)) {
+			if (ev->x > ' ') for (i = 0; i < dlg->n; i++) {
+				unsigned char *tx = _(dlg->dlg->items[i].text, term);
+				if (dlg->dlg->items[i].type == D_BUTTON && uni_upcase(GET_TERM_CHAR(term, &tx)) == uni_upcase(ev->x)) {
 					sel:
 					if (dlg->selected != i) {
 						x_display_dlg_item(dlg, &dlg->items[dlg->selected], 0);
@@ -1386,6 +1406,7 @@ clipbd_paste:
 					}
 					dlg_select_item(dlg, &dlg->items[i]);
 					goto bla;
+				}
 			}
 			if (ev->x == KBD_ENTER) for (i = 0; i < dlg->n; i++)
 				if (dlg->dlg->items[i].type == D_BUTTON && dlg->dlg->items[i].gid & B_ENTER) goto sel;
@@ -1530,9 +1551,9 @@ void draw_dlg(struct dialog_data *dlg)
 	if (!F) {
 		int i, tpos;
 		struct terminal *term = dlg->win->term;
-		fill_area(term, dlg->x, dlg->y, dlg->xw, dlg->yw, COLOR_DIALOG);
+		fill_area(term, dlg->x, dlg->y, dlg->xw, dlg->yw, ' ', COLOR_DIALOG);
 		draw_frame(term, dlg->x + DIALOG_LEFT_BORDER, dlg->y + DIALOG_TOP_BORDER, dlg->xw - 2 * DIALOG_LEFT_BORDER, dlg->yw - 2 * DIALOG_TOP_BORDER, COLOR_DIALOG_FRAME, DIALOG_FRAME);
-		i = strlen(_(dlg->dlg->title, term));
+		i = ttxtlen(term, _(dlg->dlg->title, term));
 		tpos = (dlg->xw - i) / 2;
 		if (term->spec->braille) tpos = 9;
 		print_text(term, tpos + dlg->x - 1, dlg->y + DIALOG_TOP_BORDER, 1, " ", COLOR_DIALOG_TITLE);
@@ -1544,7 +1565,7 @@ void draw_dlg(struct dialog_data *dlg)
 		struct rect r;
 		struct rect rt;
 		unsigned char *text = _(dlg->dlg->title, dlg->win->term);
-		int xtl = txtlen(text);
+		int xtl = txtlen(dlg->win->term, text);
 		int tl = xtl + 2 * G_DIALOG_TITLE_BORDER;
 		int TXT_X, TXT_Y;
 		if (tl > dlg->xw - 2 * G_DIALOG_LEFT_BORDER - 2 * G_DIALOG_VLINE_SPACE) tl = dlg->xw - 2 * G_DIALOG_LEFT_BORDER - 2 * G_DIALOG_VLINE_SPACE;
@@ -1616,14 +1637,15 @@ void max_text_width(struct terminal *term, unsigned char *text, int *width, int
 	do {
 		int c = 0;
 		while (*text && *text != '\n') {
-			if (!F) text++, c++;
-#ifdef G
+			if (!is_utf_8(term)) text++, c++;
 			else {
 				int u;
 				GET_UTF_8(text, u);
-				c += g_char_width(align & AL_MONO ? bfu_style_wb_mono : bfu_style_wb, u);
-			}
+				if (!F) c++;
+#ifdef G
+				else c += g_char_width(align & AL_MONO ? bfu_style_wb_mono : bfu_style_wb, u);
 #endif
+			}
 		}
 		if (c > *width) *width = c;
 	} while (*(text++));
@@ -1636,14 +1658,15 @@ void min_text_width(struct terminal *term, unsigned char *text, int *width, int
 	do {
 		int c = 0;
 		while (*text && *text != '\n' && *text != ' ') {
-			if (!F) text++, c++;
-#ifdef G
+			if (!is_utf_8(term)) text++, c++;
 			else {
 				int u;
 				GET_UTF_8(text, u);
-				c += g_char_width(align & AL_MONO ? bfu_style_wb_mono : bfu_style_wb, u);
-			}
+				if (!F) c++;
+#ifdef G
+				else c += g_char_width(align & AL_MONO ? bfu_style_wb_mono : bfu_style_wb, u);
 #endif
+			}
 		}
 		if (c > *width) *width = c;
 	} while (*(text++));
@@ -1658,6 +1681,45 @@ int dlg_format_text(struct dialog_data *dlg, struct terminal *term, unsigned cha
 	text = _(text, dlg->win->term);
 	if (dlg->win->term->spec->braille && !(align & AL_NOBRLEXP)) w = dlg->win->term->x;
 	if (!F) do {
+		unsigned char *t1;
+		unsigned ch;
+		int cx, lbr;
+
+		next_line:
+		t1 = text;
+		cx = 0;
+		lbr = 0;
+		next_chr:
+		ch = GET_TERM_CHAR(dlg->win->term, &t1);
+		if (ch == ' ') {
+			lbr = cx;
+		}
+		if (ch && ch != '\n') {
+			if (cx == w) {
+				if (!lbr) lbr = cx;
+				goto print_line;
+			}
+			cx++;
+			goto next_chr;
+		}
+		if (!ch && !cx) 
+			break;
+		lbr = cx;
+		print_line:
+		if (rw && lbr > *rw) *rw = lbr;
+		xx = x;
+		if ((align & AL_MASK) == AL_CENTER && !dlg->win->term->spec->braille) {
+			xx += (w - lbr) / 2;
+		}
+		for (; lbr--; xx++) {
+			ch = GET_TERM_CHAR(dlg->win->term, &text);
+			if (term) set_char(term, xx, *y, ch, co);
+		}
+		xx++;
+		if (*text == ' ' || *text == '\n') text++;
+		(*y)++;
+		goto next_line;
+#if 0
 		unsigned char *tx;
 		unsigned char *tt = text;
 		int s;
@@ -1679,11 +1741,12 @@ int dlg_format_text(struct dialog_data *dlg, struct terminal *term, unsigned cha
 				if (rw) *rw = w;
 				rw = NULL;
 			}
-			if (term) set_char(term, x + s, *y, co | *tt);
+			if (term) set_char(term, x + s, *y, *tt, co);
 			s++, tt++;
 		}
 		if (rw && xx - 1 - x > *rw) *rw = xx - 1 - x;
 		(*y)++;
+#endif
 	} while (*(text - 1));
 #ifdef G
 	else if ((tx2 = strchr(text, '\n'))) {
@@ -1741,7 +1804,7 @@ void max_buttons_width(struct terminal *term, struct dialog_item_data *butt, int
 	int w = gf_val(-2, -G_DIALOG_BUTTON_SPACE);
 	int i;
 	if (term->spec->braille) *width = term->x;
-	for (i = 0; i < n; i++) w += txtlen(_((butt++)->item->text, term)) + gf_val(6, G_DIALOG_BUTTON_SPACE + txtlen(G_DIALOG_BUTTON_L) + txtlen(G_DIALOG_BUTTON_R));
+	for (i = 0; i < n; i++) w += txtlen(term, _((butt++)->item->text, term)) + gf_val(6, G_DIALOG_BUTTON_SPACE + txtlen(term, G_DIALOG_BUTTON_L) + txtlen(term, G_DIALOG_BUTTON_R));
 	if (w > *width) *width = w;
 }
 
@@ -1750,7 +1813,7 @@ void min_buttons_width(struct terminal *term, struct dialog_item_data *butt, int
 	int i;
 	if (term->spec->braille) *width = term->x;
 	for (i = 0; i < n; i++) {
-		int w = txtlen(_((butt++)->item->text, term)) + gf_val(4, txtlen(G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R));
+		int w = txtlen(term, _((butt++)->item->text, term)) + gf_val(4, txtlen(term, G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R));
 		if (w > *width) *width = w;
 	}
 }
@@ -1777,7 +1840,7 @@ void dlg_format_buttons(struct dialog_data *dlg, struct terminal *term, struct d
 			for (i = i1; i < i2; i++) {
 				butt[i].x = p;
 				butt[i].y = *y;
-				p += (butt[i].l = txtlen(_(butt[i].item->text, dlg->win->term)) + gf_val(4, txtlen(G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R))) + gf_val(2, G_DIALOG_BUTTON_SPACE);
+				p += (butt[i].l = txtlen(dlg->win->term, _(butt[i].item->text, dlg->win->term)) + gf_val(4, txtlen(dlg->win->term, G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R))) + gf_val(2, G_DIALOG_BUTTON_SPACE);
 			}
 		}
 		*y += gf_val(2, G_BFU_FONT_SIZE * 2);
@@ -1787,7 +1850,7 @@ void dlg_format_buttons(struct dialog_data *dlg, struct terminal *term, struct d
 
 void dlg_format_checkbox(struct dialog_data *dlg, struct terminal *term, struct dialog_item_data *chkb, int x, int *y, int w, int *rw, unsigned char *text)
 {
-	int k = gf_val(4, txtlen(G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R) + G_DIALOG_CHECKBOX_SPACE);
+	int k = gf_val(4, txtlen(dlg->win->term, G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R) + G_DIALOG_CHECKBOX_SPACE);
 	if (term) {
 		chkb->x = x;
 		chkb->y = *y;
@@ -1806,10 +1869,10 @@ void dlg_format_checkboxes(struct dialog_data *dlg, struct terminal *term, struc
 	}
 }
 
-void checkboxes_width(struct terminal *term, unsigned char **texts, int *w, void (*fn)(struct terminal *, unsigned char *, int *, int))
+void checkboxes_width(struct terminal *term, unsigned char **texts, int n, int *w, void (*fn)(struct terminal *, unsigned char *, int *, int))
 {
-	int k = gf_val(4, txtlen(G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R) + G_DIALOG_CHECKBOX_SPACE);
-	while (texts[0]) {
+	int k = gf_val(4, txtlen(term, G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R) + G_DIALOG_CHECKBOX_SPACE);
+	while (n--) {
 		*w -= k;
 		fn(term, _(texts[0], term), w, 0);
 		*w += k;
@@ -1861,10 +1924,10 @@ void max_group_width(struct terminal *term, unsigned char **texts, struct dialog
 	int ww = 0;
 	if (term->spec->braille) *w = term->x;
 	while (n--) {
-		int wx = item->item->type == D_CHECKBOX ? gf_val(4, txtlen(G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R) + G_DIALOG_CHECKBOX_SPACE) :
-			item->item->type == D_BUTTON ? txtlen(_(item->item->text, term)) + (gf_val(4, txtlen(G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R))) :
+		int wx = item->item->type == D_CHECKBOX ? gf_val(4, txtlen(term, G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R) + G_DIALOG_CHECKBOX_SPACE) :
+			item->item->type == D_BUTTON ? txtlen(term, _(item->item->text, term)) + (gf_val(4, txtlen(term, G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R))) :
 			gf_val(item->item->dlen + 1, (item->item->dlen + 1) * G_DIALOG_FIELD_WIDTH);
-		wx += txtlen(_(texts[0], term));
+		wx += txtlen(term, _(texts[0], term));
 		if (n) gf_val(wx++, wx += G_DIALOG_GROUP_SPACE);
 		ww += wx;
 		texts++;
@@ -1877,10 +1940,10 @@ void min_group_width(struct terminal *term, unsigned char **texts, struct dialog
 {
 	if (term->spec->braille) *w = term->x;
 	while (n--) {
-		int wx = item->item->type == D_CHECKBOX ? gf_val(4, txtlen(G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R) + G_DIALOG_CHECKBOX_SPACE) :
-			item->item->type == D_BUTTON ? txtlen(_(item->item->text, term)) + (gf_val(4, txtlen(G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R))) :
+		int wx = item->item->type == D_CHECKBOX ? gf_val(4, txtlen(term, G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R) + G_DIALOG_CHECKBOX_SPACE) :
+			item->item->type == D_BUTTON ? txtlen(term, _(item->item->text, term)) + (gf_val(4, txtlen(term, G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R))) :
 			gf_val(item->item->dlen + 1, (item->item->dlen + 1) * G_DIALOG_FIELD_WIDTH);
-		wx += txtlen(_(texts[0], term));
+		wx += txtlen(term, _(texts[0], term));
 		if (wx > *w) *w = wx;
 		texts++;
 		item++;
@@ -1893,11 +1956,11 @@ void dlg_format_group(struct dialog_data *dlg, struct terminal *term, unsigned c
 	int nx = 0;
 	if (dlg->win->term->spec->braille) w = dlg->win->term->x;
 	while (n--) {
-		int wx = item->item->type == D_CHECKBOX ? gf_val(3, txtlen(G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R)) :
-			item->item->type == D_BUTTON ? txtlen(_(item->item->text, dlg->win->term)) + (gf_val(4, txtlen(G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R))) :
+		int wx = item->item->type == D_CHECKBOX ? gf_val(3, txtlen(dlg->win->term, G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R)) :
+			item->item->type == D_BUTTON ? txtlen(dlg->win->term, _(item->item->text, dlg->win->term)) + (gf_val(4, txtlen(dlg->win->term, G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R))) :
 			gf_val(item->item->dlen, item->item->dlen * G_DIALOG_FIELD_WIDTH);
 		int sl;
-		if (_(texts[0], dlg->win->term)[0]) sl = txtlen(_(texts[0], dlg->win->term)) + gf_val(1, G_DIALOG_GROUP_TEXT_SPACE);
+		if (_(texts[0], dlg->win->term)[0]) sl = txtlen(dlg->win->term, _(texts[0], dlg->win->term)) + gf_val(1, G_DIALOG_GROUP_TEXT_SPACE);
 		else sl = 0;
 		wx += sl;
 		if (dlg->win->term->spec->braille) {
@@ -1910,11 +1973,11 @@ void dlg_format_group(struct dialog_data *dlg, struct terminal *term, unsigned c
 			(*y) += gf_val(2, G_BFU_FONT_SIZE * 2);
 		}
 		if (term) {
-			if (!F) print_text(term, x + nx + 4 * (item->item->type == D_CHECKBOX), *y, strlen(_(texts[0], dlg->win->term)), _(texts[0], dlg->win->term), COLOR_DIALOG_TEXT);
+			if (!F) print_text(term, x + nx + 4 * (item->item->type == D_CHECKBOX), *y, ttxtlen(term, _(texts[0], dlg->win->term)), _(texts[0], dlg->win->term), COLOR_DIALOG_TEXT);
 #ifdef G
 			else {
 				int l, ll;
-				l = ll = x + nx + (item->item->type == D_CHECKBOX ? txtlen(G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R) + G_DIALOG_GROUP_TEXT_SPACE : 0);
+				l = ll = x + nx + (item->item->type == D_CHECKBOX ? txtlen(dlg->win->term, G_DIALOG_CHECKBOX_L G_DIALOG_CHECKBOX_X G_DIALOG_CHECKBOX_R) + G_DIALOG_GROUP_TEXT_SPACE : 0);
 				g_print_text(drv, term->dev, ll, *y, bfu_style_bw, _(texts[0], dlg->win->term), &ll);
 				exclude_from_set(&dlg->s, l, *y, ll, *y + G_BFU_FONT_SIZE);
 			}
@@ -1936,31 +1999,33 @@ void dlg_format_group(struct dialog_data *dlg, struct terminal *term, unsigned c
 void checkbox_list_fn(struct dialog_data *dlg)
 {
 	struct terminal *term = dlg->win->term;
+	int n_checkboxes;
 	int max = 0, min = 0;
 	int w, rw;
 	int y = 0;
-	checkboxes_width(term, dlg->dlg->udata, &max, max_text_width);
-	checkboxes_width(term, dlg->dlg->udata, &min, min_text_width);
-	max_buttons_width(term, dlg->items + dlg->n - 2, 2, &max);
-	min_buttons_width(term, dlg->items + dlg->n - 2, 2, &min);
+	for (n_checkboxes = 0; ((unsigned char **)dlg->dlg->udata)[n_checkboxes]; n_checkboxes++) ;
+	checkboxes_width(term, dlg->dlg->udata, n_checkboxes, &max, max_text_width);
+	checkboxes_width(term, dlg->dlg->udata, n_checkboxes, &min, min_text_width);
+	max_buttons_width(term, dlg->items + n_checkboxes, dlg->n - n_checkboxes, &max);
+	min_buttons_width(term, dlg->items + n_checkboxes, dlg->n - n_checkboxes, &min);
 	w = term->x * 9 / 10 - 2 * DIALOG_LB;
 	if (w > max) w = max;
 	if (w < min) w = min;
 	if (w > term->x - 2 * DIALOG_LB) w = term->x - 2 * DIALOG_LB;
 	if (w < 5) w = 5;
 	rw = 0;
-	dlg_format_checkboxes(dlg, NULL, dlg->items, dlg->n - 2, 0, &y, w, &rw, dlg->dlg->udata);
+	dlg_format_checkboxes(dlg, NULL, dlg->items, n_checkboxes, 0, &y, w, &rw, dlg->dlg->udata);
 	y += LL;
-	dlg_format_buttons(dlg, NULL, dlg->items + dlg->n - 2, 2, 0, &y, w, &rw, AL_CENTER);
+	dlg_format_buttons(dlg, NULL, dlg->items + n_checkboxes, dlg->n - n_checkboxes, 0, &y, w, &rw, AL_CENTER);
 	w = rw;
 	dlg->xw = rw + 2 * DIALOG_LB;
 	dlg->yw = y + 2 * DIALOG_TB;
 	center_dlg(dlg);
 	draw_dlg(dlg);
 	y = dlg->y + DIALOG_TB + LL;
-	dlg_format_checkboxes(dlg, term, dlg->items, dlg->n - 2, dlg->x + DIALOG_LB, &y, w, NULL, dlg->dlg->udata);
+	dlg_format_checkboxes(dlg, term, dlg->items, n_checkboxes, dlg->x + DIALOG_LB, &y, w, NULL, dlg->dlg->udata);
 	y += LL;
-	dlg_format_buttons(dlg, term, dlg->items + dlg->n - 2, 2, dlg->x + DIALOG_LB, &y, w, &rw, AL_CENTER);
+	dlg_format_buttons(dlg, term, dlg->items + n_checkboxes, dlg->n - n_checkboxes, dlg->x + DIALOG_LB, &y, w, &rw, AL_CENTER);
 }
 
 void group_fn(struct dialog_data *dlg)
@@ -2028,7 +2093,7 @@ void msg_box_fn(struct dialog_data *dlg)
 	mem_free(text);
 }
 
-int msg_box_button(struct dialog_data *dlg, struct dialog_item_data *di)
+static int msg_box_button(struct dialog_data *dlg, struct dialog_item_data *di)
 {
 	void (*fn)(void *) = (void (*)(void *))di->item->udata;
 	void *data = dlg->dlg->udata2;
@@ -2128,7 +2193,7 @@ void add_to_history(struct history *h, unsigned char *t)
 	}
 }
 
-int input_field_cancel(struct dialog_data *dlg, struct dialog_item_data *di)
+static int input_field_cancel(struct dialog_data *dlg, struct dialog_item_data *di)
 {
 	void (*fn)(void *) = di->item->udata;
 	void *data = dlg->dlg->udata2;
@@ -2137,7 +2202,7 @@ int input_field_cancel(struct dialog_data *dlg, struct dialog_item_data *di)
 	return 0;
 }
 
-int input_field_ok(struct dialog_data *dlg, struct dialog_item_data *di)
+static int input_field_ok(struct dialog_data *dlg, struct dialog_item_data *di)
 {
 	void (*fn)(void *, unsigned char *) = di->item->udata;
 	void *data = dlg->dlg->udata2;
