@@ -23,7 +23,7 @@ struct codepage_desc {
 #include "entity.inc"
 #include "upcase.inc"
 
-char strings[256][2] = {
+static unsigned char strings[256][2] = {
 	"\000", "\001", "\002", "\003", "\004", "\005", "\006", "\007",
 	"\010", "\011", "\012", "\013", "\014", "\015", "\016", "\017",
 	"\020", "\021", "\022", "\023", "\024", "\025", "\026", "\033",
@@ -58,24 +58,16 @@ char strings[256][2] = {
 	"\370", "\371", "\372", "\373", "\374", "\375", "\376", "\377",
 };
 
-/* prototypes */
-void free_translation_table(struct conv_table *);
-void new_translation_table(struct conv_table *);
-void add_utf_8(struct conv_table *, int, unsigned char *);
-void free_utf_table(void);
-struct conv_table *get_translation_table_to_utf_8(int);
-
-
-void free_translation_table(struct conv_table *p)
+static void free_translation_table(struct conv_table *p)
 {
 	int i;
 	for (i = 0; i < 256; i++) if (p[i].t) free_translation_table(p[i].u.tbl);
 	mem_free(p);
 }
 
-unsigned char *no_str = "*";
+static unsigned char *no_str = "*";
 
-void new_translation_table(struct conv_table *p)
+static void new_translation_table(struct conv_table *p)
 {
 	int i;
 	for (i = 0; i < 256; i++) if (p[i].t) free_translation_table(p[i].u.tbl);
@@ -83,7 +75,7 @@ void new_translation_table(struct conv_table *p)
 	for (; i < 256; i++) p[i].t = 0, p[i].u.str = no_str;
 }
 
-int strange_chars[32] = {
+static int strange_chars[32] = {
 	0x20ac, 0x0000, 0x002a, 0x0000, 0x201e, 0x2026, 0x2020, 0x2021,
 	0x005e, 0x2030, 0x0160, 0x003c, 0x0152, 0x0000, 0x0000, 0x0000,
 	0x0000, 0x0060, 0x0027, 0x0022, 0x0022, 0x002a, 0x2013, 0x2014,
@@ -93,7 +85,7 @@ int strange_chars[32] = {
 #define U_EQUAL(a, b) unicode_7b[a].x == (b)
 #define U_ABOVE(a, b) unicode_7b[a].x > (b)
 
-static inline unsigned char *u2cp(int u, int to, int fallback)
+unsigned char *u2cp(int u, int to, int fallback)
 {
 	int j, s;
 	again:
@@ -115,15 +107,16 @@ static inline unsigned char *u2cp(int u, int to, int fallback)
 	return NULL;
 }
 
-int cp2u(unsigned char ch, int from)
+int cp2u(unsigned ch, int from)
 {
 	struct table_entry *e;
+	if (from == utf8_table) return ch;
 	if (from < 0 || ch < 0x80) return ch;
 	for (e = codepages[from].table; e->c; e++) if (e->c == ch) return e->u;
 	return -1;
 }
 
-unsigned char utf_buffer[7];
+static unsigned char utf_buffer[7];
 
 unsigned char *encode_utf_8(int u)
 {
@@ -156,7 +149,7 @@ unsigned char *encode_utf_8(int u)
 	return utf_buffer;
 }
 
-void add_utf_8(struct conv_table *ct, int u, unsigned char *str)
+static void add_utf_8(struct conv_table *ct, int u, unsigned char *str)
 {
 	unsigned char *p = encode_utf_8(u);
 	while (p[1]) {
@@ -183,16 +176,16 @@ void add_utf_8(struct conv_table *ct, int u, unsigned char *str)
 	if (ct[*p].u.str == no_str) ct[*p].u.str = str;
 }
 
-struct conv_table utf_table[256];
-int utf_table_init = 1;
+static struct conv_table utf_table[256];
+static int utf_table_init = 1;
 
-void free_utf_table(void)
+static void free_utf_table(void)
 {
 	int i;
 	for (i = 128; i < 256; i++) mem_free(utf_table[i].u.str);
 }
 
-struct conv_table *get_translation_table_to_utf_8(int from)
+static struct conv_table *get_translation_table_to_utf_8(int from)
 {
 	int i;
 	static int lfr = -1;
@@ -290,8 +283,8 @@ int get_utf_8(unsigned char **s)
 	return v;
 }
 
-struct conv_table table[256];
-int table_init = 1;
+static struct conv_table table[256];
+static int table_init = 1;
 
 void free_conv_table(void)
 {
@@ -312,11 +305,11 @@ struct conv_table *get_translation_table(int from, int to)
 	new_translation_table(table);
 	if (codepages[from].table == table_utf_8) {
 		int j;
-		for (j = 0; codepages[to].table[j].c; j++) add_utf_8(table, codepages[to].table[j].u, codepages[to].table[j].u == 0xa0 ? "\001" : codepages[to].table[j].u == 0xad ? "" : strings[codepages[to].table[j].c]);
+		for (j = 0; codepages[to].table[j].c; j++) add_utf_8(table, codepages[to].table[j].u, codepages[to].table[j].u == 0xa0 ? (unsigned char *)"\001" : codepages[to].table[j].u == 0xad ? (unsigned char *)"" : strings[codepages[to].table[j].c]);
 		for (i = 0; unicode_7b[i].x != -1; i++) if (unicode_7b[i].x >= 0x80) add_utf_8(table, unicode_7b[i].x, unicode_7b[i].s);
 	} else for (i = 128; i < 256; i++) {
 		int j;
-		char *u;
+		unsigned char *u;
 		for (j = 0; codepages[from].table[j].c; j++) {
 			if (codepages[from].table[j].c == i) goto f;
 		}
@@ -343,18 +336,18 @@ int get_entity_number(unsigned char *st, int l)
 	int n = 0;
 	if (upcase(st[0]) == 'X') {
 		st++, l--;
-		if (!l || l > 4) return -1;
+		if (!l) return -1;
 		do {
-			char c = upcase(*(st++));
+			unsigned char c = upcase(*(st++));
 			if (c >= '0' && c <= '9') n = n * 16 + c - '0';
 			else if (c >= 'A' && c <= 'F') n = n * 16 + c - 'A' + 10;
 			else return -1;
 			if (n >= 0x10000) return -1;
 		} while (--l);
 	} else {
-		if (!l || l > 5) return -1;
+		if (!l) return -1;
 		do {
-			char c = *(st++);
+			unsigned char c = *(st++);
 			if (c >= '0' && c <= '9') n = n * 10 + c - '0';
 			else return -1;
 			if (n >= 0x10000) return -1;
@@ -520,15 +513,10 @@ unsigned char *get_cp_mime_name(int index)
 	return codepages[index].aliases[0];
 }
 
-int is_cp_special(int index)
-{
-	return codepages[index].table == table_utf_8;
-}
-
 #define UP_EQUAL(a, b) unicode_upcase[a].lo == (b)
 #define UP_ABOVE(a, b) unicode_upcase[a].lo > (b)
 
-unsigned char charset_upcase(unsigned char ch, int cp)
+unsigned charset_upcase(unsigned ch, int cp)
 {
 	unsigned u;
 	int res;
@@ -537,11 +525,19 @@ unsigned char charset_upcase(unsigned char ch, int cp)
 	u = cp2u(ch, cp);
 	BIN_SEARCH(sizeof(unicode_upcase) / sizeof(*unicode_upcase), UP_EQUAL, UP_ABOVE, u, res);
 	if (res == -1) return ch;
+	if (codepages[cp].table == table_utf_8) return unicode_upcase[res].up;
 	str = u2cp(unicode_upcase[res].up, cp, 0);
 	if (!str || !str[0] || str[1]) return ch;
 	return str[0];
 }
 
+unsigned uni_upcase(unsigned ch)
+{
+	int cp = get_cp_index("utf-8");
+	if (cp < 0) cp = 0;
+	return charset_upcase(ch, cp);
+}
+
 void charset_upcase_string(unsigned char **chp, int cp)
 {
 	unsigned char *ch = *chp;
@@ -608,3 +604,35 @@ int compare_case_utf8(unsigned char *u1, unsigned char *u2)
 	}
 }
 
+int strlen_utf8(unsigned char *s)
+{
+	int len = 0;
+	while (1) {
+		unsigned c;
+		GET_UTF_8(s, c);
+		if (!c) return len;
+		len++;
+	}
+}
+
+int cp_len(int cp, unsigned char *s)
+{
+	if (is_cp_special(cp)) return strlen_utf8(s);
+	return strlen((char *)s);
+}
+
+unsigned char *cp_strchr(int charset, unsigned char *str, unsigned chr)
+{
+	if (!is_cp_special(charset)) {
+		if (chr >= 0x100)
+			return NULL;
+		return (unsigned char *)strchr(str, chr);
+	}
+	while (1) {
+		unsigned char *o_str = str;
+		unsigned c;
+		GET_UTF_8(str, c);
+		if (!c) return NULL;
+		if (c == chr) return o_str;
+	}
+}
