@@ -9,12 +9,6 @@
 #define LOG_TRANSFER	"/tmp/log"
 */
 
-/* prototypes */
-void ssl_want_read(struct connection *);
-void write_select(struct connection *);
-void read_select(struct connection *);
-
-
 #ifdef LOG_TRANSFER
 static void log_data(unsigned char *data, int len)
 {
@@ -145,13 +139,15 @@ int get_pasv_socket(struct connection *c, int cc, int *sock, unsigned char *port
 }
 
 #ifdef HAVE_SSL
-void ssl_want_read(struct connection *c)
+static void ssl_want_read(struct connection *c)
 {
 	struct conn_info *b = c->newconn;
 
 	set_timeout(c);
 
+#ifndef HAVE_NSS
 	if (c->no_tsl) c->ssl->options |= SSL_OP_NO_TLSv1;
+#endif
 	switch (SSL_get_error(c->ssl, SSL_connect(c->ssl))) {
 		case SSL_ERROR_NONE:
 			c->newconn = NULL;
@@ -196,6 +192,7 @@ void handle_socks(struct connection *c)
 		return;
 	}
 	add_to_str(&command, &len, host);
+	add_to_str(&command, &len, proxies.dns_append);
 	add_chr_to_str(&command, &len, 0);
 	mem_free(host);
 	wr = write(*b->sock, command + b->socks_byte_count, len - b->socks_byte_count);
@@ -317,8 +314,13 @@ void connected(struct connection *c)
 #ifdef HAVE_SSL
 	if (c->ssl) {
 		c->ssl = getSSL();
+		if (!c->ssl) {
+			goto ssl_error;
+		}
 		SSL_set_fd(c->ssl, *b->sock);
+#ifndef HAVE_NSS
 		if (c->no_tsl) c->ssl->options |= SSL_OP_NO_TLSv1;
+#endif
 		switch (SSL_get_error(c->ssl, SSL_connect(c->ssl))) {
 			case SSL_ERROR_WANT_READ:
 				setcstate(c, S_SSL_NEG);
@@ -331,6 +333,7 @@ void connected(struct connection *c)
 			case SSL_ERROR_NONE:
 				break;
 			default:
+			ssl_error:
 				c->no_tsl++;
 				setcstate(c, S_SSL_ERROR);
 				retry_connection(c);
@@ -351,7 +354,7 @@ struct write_buffer {
 	unsigned char data[1];
 };
 
-void write_select(struct connection *c)
+static void write_select(struct connection *c)
 {
 	struct write_buffer *wb;
 	int wr;
@@ -381,6 +384,12 @@ void write_select(struct connection *c)
 	} else
 #endif
 		if ((wr = write(wb->sock, wb->data + wb->pos, wb->len - wb->pos)) <= 0) {
+#ifdef ATHEOS
+	/* Workaround for a bug in Syllable */
+			if (wr && errno == EAGAIN) {
+				return;
+			}
+#endif
 			setcstate(c, wr ? -errno : S_CANT_WRITE);
 			retry_connection(c);
 			return;
@@ -414,7 +423,7 @@ void write_to_socket(struct connection *c, int s, unsigned char *data, int len,
 
 #define READ_SIZE	64240
 
-void read_select(struct connection *c)
+static void read_select(struct connection *c)
 {
 	struct read_buffer *rb;
 	int rd;
@@ -462,11 +471,7 @@ void read_select(struct connection *c)
    Turn off compression support once before the final retry.
 */
 				unsigned char *prot, *h;
-				int is_restartable;
-				c->tries++;
-				is_restartable = is_connection_restartable(c) && c->tries < 10;
-				c->tries--;
-				if (!is_restartable && (prot = get_protocol_name(c->url))) {
+				if (is_last_try(c) && (prot = get_protocol_name(c->url))) {
 					if (!strcasecmp(prot, "http")) {
 						if ((h = get_host_name(c->url))) {
 							add_blacklist_entry(h, BL_NO_COMPRESSION);
