@@ -27,18 +27,6 @@ struct c_server {
 
 struct list_head c_servers = { &c_servers, &c_servers };
 
-void accept_cookie(struct cookie *);
-void delete_cookie(struct cookie *);
-
-/* prototypes */
-int check_domain_security(unsigned char *, unsigned char *);
-struct cookie *find_cookie_id(void *);
-void reject_cookie(void *);
-void cookie_default(void *, int);
-void accept_cookie_always(void *);
-void accept_cookie_never(void *);
-
-
 void free_cookie(struct cookie *c)
 {
 	mem_free(c->name);
@@ -48,28 +36,43 @@ void free_cookie(struct cookie *c)
 	if (c->domain) mem_free(c->domain);
 }
 
-int check_domain_security(unsigned char *server, unsigned char *domain)
+static int check_domain_security(unsigned char *server, unsigned char *domain)
 {
 	size_t i, j, dl;
 	int nd;
+	unsigned char *tld[] = { ".com", ".edu", ".net", ".org", ".gov", ".mil", ".int", NULL };
 	if (domain[0] == '.') domain++;
 	dl = strlen(domain);
 	if (dl > strlen(server)) return 1;
 	for (i = strlen(server) - dl, j = 0; server[i]; i++, j++)
 		if (upcase(server[i]) != upcase(domain[j])) return 1;
 	nd = 2;
-	if (dl > 4 && domain[dl - 4] == '.') {
-		unsigned char *tld[] = { "com", "edu", "net", "org", "gov", "mil", "int", NULL };
-		for (i = 0; tld[i]; i++) if (!casecmp(tld[i], &domain[dl - 3], 3)) {
+	for (i = 0; tld[i]; i++) {
+		size_t tl = strlen(tld[i]);
+		if (dl > tl && !casecmp(tld[i], &domain[dl - tl], tl)) {
 			nd = 1;
 			break;
 		}
 	}
+	if (nd == 2) {
+		unsigned char *last_dot = strrchr(domain, '.');
+		i = 0;
+		if (last_dot) {
+			while (last_dot > domain) {
+				last_dot--;
+				if (*last_dot == '.') break;
+				i++;
+			}
+		}
+		if (i >= 4) nd = 1;
+	}
 	for (i = 0; domain[i]; i++) if (domain[i] == '.') if (!--nd) break;
 	if (nd > 0) return 1;
 	return 0;
 }
 
+static void accept_cookie(struct cookie *);
+
 /* sezere 1 cookie z retezce str, na zacatku nesmi byt zadne whitechars
  * na konci muze byt strednik nebo 0
  * cookie musi byt ve tvaru nazev=hodnota, kolem rovnase nesmi byt zadne mezery
@@ -80,7 +83,7 @@ int set_cookie(struct terminal *term, unsigned char *url, unsigned char *str)
 	int noval = 0;
 	struct cookie *cookie;
 	struct c_server *cs;
-	unsigned char *p, *q, *s, *server, *date, *document;
+	unsigned char *p, *q, *s, *server, *date;
 	if (accept_cookies == ACCEPT_NONE) return 0;
 	for (p = str; *p != ';' && *p; p++) /*if (WHITECHAR(*p)) return 0*/;
 	for (q = str; *q != '='; q++) if (!*q || q >= p) {
@@ -89,12 +92,11 @@ int set_cookie(struct terminal *term, unsigned char *url, unsigned char *str)
 	}
 	if (str == q || q + 1 == p) return 0;
 	cookie = mem_alloc(sizeof(struct cookie));
-	document = get_url_data(url);
 	server = get_host_name(url);
 	cookie->name = memacpy(str, q - str);
 	cookie->value = !noval ? memacpy(q + 1, p - q - 1) : NULL;
 	cookie->server = stracpy(server);
-	date = parse_header_param(str, "expires");
+	date = parse_header_param(str, "expires", 0);
 	if (date) {
 		cookie->expires = parse_http_date(date);
 		/* kdo tohle napsal a proc ?? */
@@ -102,7 +104,7 @@ int set_cookie(struct terminal *term, unsigned char *url, unsigned char *str)
 		mem_free(date);
 	} else
 		cookie->expires = 0;
-	if (!(cookie->path = parse_header_param(str, "path"))) {
+	if (!(cookie->path = parse_header_param(str, "path", 0))) {
 		/*unsigned char *w;*/
 		cookie->path = stracpy("/");
 		/*
@@ -124,9 +126,9 @@ int set_cookie(struct terminal *term, unsigned char *url, unsigned char *str)
 			cookie->path[0] = '/';
 		}
 	}
-	if (!(cookie->domain = parse_header_param(str, "domain"))) cookie->domain = stracpy(server);
+	if (!(cookie->domain = parse_header_param(str, "domain", 0))) cookie->domain = stracpy(server);
 	if (cookie->domain[0] == '.') memmove(cookie->domain, cookie->domain + 1, strlen(cookie->domain));
-	if ((s = parse_header_param(str, "secure"))) {
+	if ((s = parse_header_param(str, "secure", 0))) {
 		cookie->secure = 1;
 		mem_free(s);
 	} else cookie->secure = 0;
@@ -156,7 +158,7 @@ int set_cookie(struct terminal *term, unsigned char *url, unsigned char *str)
 	return 0;
 }
 
-void accept_cookie(struct cookie *c)
+static void accept_cookie(struct cookie *c)
 {
 	struct c_domain *cd;
 	struct cookie *d, *e;
@@ -179,7 +181,8 @@ void accept_cookie(struct cookie *c)
 	add_to_list(c_domains, cd);
 }
 
-void delete_cookie(struct cookie *c)
+#if 0
+static void delete_cookie(struct cookie *c)
 {
 	struct c_domain *cd;
 	struct cookie *d;
@@ -195,7 +198,7 @@ void delete_cookie(struct cookie *c)
 	mem_free(c);
 }
 
-struct cookie *find_cookie_id(void *idp)
+static struct cookie *find_cookie_id(void *idp)
 {
 	long id = (my_intptr_t)idp;
 	struct cookie *c;
@@ -203,14 +206,14 @@ struct cookie *find_cookie_id(void *idp)
 	return NULL;
 }
 
-void reject_cookie(void *idp)
+static void reject_cookie(void *idp)
 {
 	struct cookie *c;
 	if (!(c = find_cookie_id(idp))) return;
 	delete_cookie(c);
 }
 
-void cookie_default(void *idp, int a)
+static void cookie_default(void *idp, int a)
 {
 	struct cookie *c;
 	struct c_server *s;
@@ -223,16 +226,17 @@ void cookie_default(void *idp, int a)
 	s->accept = a;
 }
 
-void accept_cookie_always(void *idp)
+static void accept_cookie_always(void *idp)
 {
 	cookie_default(idp, 1);
 }
 
-void accept_cookie_never(void *idp)
+static void accept_cookie_never(void *idp)
 {
 	cookie_default(idp, 0);
 	reject_cookie(idp);
 }
+#endif
 
 int is_in_domain(unsigned char *d, unsigned char *s)
 {
@@ -259,7 +263,7 @@ int cookie_expired(struct cookie *c)	/* parse_http_date is broken */
 	return 0 && (c->expires && c->expires < time(NULL));
 }
 
-void send_cookies(unsigned char **s, int *l, unsigned char *url)
+void add_cookies(unsigned char **s, int *l, unsigned char *url)
 {
 	int nc = 0;
 	struct c_domain *cd;
@@ -271,7 +275,7 @@ void send_cookies(unsigned char **s, int *l, unsigned char *url)
 	mem_free(server);
 	return;
 	ok:
-	foreach (c, cookies) if (is_in_domain(c->domain, server)) if (is_path_prefix(c->path, data)) {
+	foreachback (c, cookies) if (is_in_domain(c->domain, server)) if (is_path_prefix(c->path, data)) {
 		if (cookie_expired(c)) {
 			d = c;
 			c = c->prev;
