@@ -11,93 +11,14 @@ struct list_head html_stack = {&html_stack, &html_stack};
 
 int html_format_changed = 0;
 
-/* prototypes */
-unsigned char *get_url_val(unsigned char *, unsigned char *);
-unsigned char *get_exact_attr_val(unsigned char *, unsigned char *);
-void roman(char *, unsigned);
-unsigned char *get_target(unsigned char *);
-void debug_stack(void);
-void get_js_event(unsigned char *, unsigned char *, unsigned char **);
-int get_js_events_x(struct js_event_spec **spec, unsigned char *a);
-int get_js_events(unsigned char *);
-void kill_until(int, ...);
-int parse_width(unsigned char *, int);
-void put_link_line(unsigned char *, unsigned char *, unsigned char *, unsigned char *);
-void html_span(unsigned char *);
-void html_bold(unsigned char *);
-void html_italic(unsigned char *);
-void html_underline(unsigned char *);
-void html_fixed(unsigned char *);
-void html_invert(unsigned char *);
-void html_a(unsigned char *);
-void html_a_special(unsigned char *, unsigned char *, unsigned char *);
-void html_sub(unsigned char *);
-void html_sup(unsigned char *);
-void html_font(unsigned char *);
-void html_img(unsigned char *);
-void html_obj(unsigned char *, int);
-void html_embed(unsigned char *);
-void html_object(unsigned char *);
-void html_body(unsigned char *);
-void html_skip(unsigned char *);
-void html_title(unsigned char *);
-void html_script(unsigned char *);
-void html_noscript(unsigned char *);
-void html_center(unsigned char *);
-void html_linebrk(unsigned char *);
-void html_br(unsigned char *);
-void html_form(unsigned char *);
-void html_p(unsigned char *);
-void html_address(unsigned char *);
-void html_blockquote(unsigned char *);
-void html_h(int , char *);
-void html_h1(unsigned char *);
-void html_h2(unsigned char *);
-void html_h3(unsigned char *);
-void html_h4(unsigned char *);
-void html_h5(unsigned char *);
-void html_h6(unsigned char *);
-void html_pre(unsigned char *);
-void html_hr(unsigned char *);
-void html_table(unsigned char *);
-void html_tr(unsigned char *);
-void html_th(unsigned char *);
-void html_td(unsigned char *);
-void html_base(unsigned char *);
-void html_ul(unsigned char *);
-void html_ol(unsigned char *);
-void html_li(unsigned char *);
-void html_dl(unsigned char *);
-void html_dt(unsigned char *);
-void html_dd(unsigned char *);
-void get_html_form(unsigned char *, struct form *);
-void find_form_for_input(unsigned char *);
-void html_button(unsigned char *);
-void html_input(unsigned char *);
-void html_select(unsigned char *);
-void html_option(unsigned char *);
-void new_menu_item(unsigned char *, long, int);
-void init_menu(void);
-struct menu_item *detach_menu(void);
-void destroy_menu(void);
-int menu_contains(struct menu_item *, int);
-int do_html_select(unsigned char *, unsigned char *, unsigned char *, unsigned char **, void *);
-void html_textarea(unsigned char *);
-void do_html_textarea(unsigned char *, unsigned char *, unsigned char *, unsigned char **, void *);
-void html_iframe(unsigned char *);
-void html_noframes(unsigned char *);
-void html_frame(unsigned char *);
-void parse_frame_widths(unsigned char *, int, int, int **, int *);
-void html_frameset(unsigned char *);
-void html_link(unsigned char *);
-void process_head(unsigned char *);
-int qd(unsigned char *, unsigned char *, int *);
-void scan_area_tag(unsigned char *, unsigned char *, unsigned char **, struct memory_list **);
-void menu_labels(struct menu_item *, unsigned char *, unsigned char **);
+static inline int isA(unsigned char c)
+{
+	return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z');
+}
 
 static inline int atchr(unsigned char c)
 {
-	return /*isA(c) ||*/ (c > ' ' && c != '=' && c != '<' && c != '>' && c != '\\');
+	return /*isA(c) ||*/ (c > ' ' && c != '=' && c != '<' && c != '>');
 }
 
 /* accepts one html element */
@@ -115,11 +36,10 @@ int parse_element(unsigned char *e, unsigned char *eof, unsigned char **name, in
 	if (*e == '/') {
 		e++;
 		if (*e == '>' || *e == '<') goto xx;
+	} else if (!isA(*e)) {
+		return -1;
 	}
-	if (!isA(*e)) return -1;
-	goto x1;
-	while (isA(*e) || *e == '=') {
-		x1:
+	while (isA(*e) || (*e >= '0' && *e <= '9') || *e == '_' || *e == '-' || *e == '=') {
 		e++;
 		if (e >= eof) return -1;
 	}
@@ -221,7 +141,7 @@ unsigned char *get_attr_val(unsigned char *e, unsigned char *name)
 			e++;
 		}
 	} else {
-		char uu = *e;
+		unsigned char uu = *e;
 		/*a:*/
 		e++;
 		while (*e != uu) {
@@ -270,14 +190,14 @@ unsigned char *get_attr_val(unsigned char *e, unsigned char *name)
 
 int has_attr(unsigned char *e, unsigned char *name)
 {
-	char *a;
+	unsigned char *a;
 	if (!(a = get_attr_val(e, name))) return 0;
 	mem_free(a);
 	return 1;
 }
 
 /*
-unsigned char *get_url_val(unsigned char *e, unsigned char *name)
+static unsigned char *get_url_val(unsigned char *e, unsigned char *name)
 {
 	int n = 0;
 	unsigned char *p, *q, *pp;
@@ -293,7 +213,7 @@ unsigned char *get_url_val(unsigned char *e, unsigned char *name)
 }
 */
 
-unsigned char *get_url_val(unsigned char *e, unsigned char *name)
+static unsigned char *get_url_val(unsigned char *e, unsigned char *name)
 {
 	unsigned char *a;
 	get_attr_val_nl = 1;
@@ -302,7 +222,7 @@ unsigned char *get_url_val(unsigned char *e, unsigned char *name)
 	return a;
 }
 
-unsigned char *get_exact_attr_val(unsigned char *e, unsigned char *name)
+static unsigned char *get_exact_attr_val(unsigned char *e, unsigned char *name)
 {
 	unsigned char *a;
 	get_attr_val_nl = 2;
@@ -325,7 +245,7 @@ unsigned char *get_exact_attr_val(unsigned char *e, unsigned char *name)
 
 struct {
 	unsigned short int n;
-	char *s;
+	unsigned char *s;
 } roman_tbl[] = {
 	{1000,	"m"},
 	{999,	"im"},
@@ -355,7 +275,7 @@ struct {
 	{0,	NULL}
 };
 
-void roman(char *p, unsigned n)
+static void roman(char *p, unsigned n)
 {
 	int i = 0;
 	if (n >= 4000) {
@@ -381,7 +301,7 @@ void roman(char *p, unsigned n)
 }
 
 struct color_spec {
-	char *name;
+	unsigned char *name;
 	int rgb;
 };
 
@@ -559,7 +479,7 @@ found:
 
 int get_color(unsigned char *a, unsigned char *c, struct rgb *rgb)
 {
-	char *at;
+	unsigned char *at;
 	int r = -1;
 	if (d_opt->col >= 1 || F) if ((at = get_attr_val(a, c))) {
 		r = decode_color(at, rgb);
@@ -574,7 +494,7 @@ int get_bgcolor(unsigned char *a, struct rgb *rgb)
 	return get_color(a, "bgcolor", rgb);
 }
 
-unsigned char *get_target(unsigned char *a)
+static unsigned char *get_target(unsigned char *a)
 {
 	return get_attr_val(a, "target");
 }
@@ -611,7 +531,7 @@ static inline void kill_elem(char *e)
 		kill_html_stack_item(&html_top);
 }
 
-#ifdef DEBUG
+#if defined(DEBUG) && 0
 void debug_stack(void)
 {
 	struct html_element *e;
@@ -663,7 +583,7 @@ void html_stack_dup(void)
 
 
 #ifdef JS
-void get_js_event(unsigned char *a, unsigned char *name, unsigned char **where)
+static void get_js_event(unsigned char *a, unsigned char *name, unsigned char **where)
 {
 	unsigned char *v;
 	if ((v = get_attr_val(a, name))) {
@@ -672,7 +592,7 @@ void get_js_event(unsigned char *a, unsigned char *name, unsigned char **where)
 	}
 }
 
-int get_js_events_x(struct js_event_spec **spec, unsigned char *a)
+static int get_js_events_x(struct js_event_spec **spec, unsigned char *a)
 {
 	if (!has_attr(a, "onkeyup") && !has_attr(a, "onkeydown") && !has_attr(a,"onkeypress") && !has_attr(a,"onchange") && !has_attr(a, "onfocus") && !has_attr(a,"onblur") && !has_attr(a, "onclick") && !has_attr(a, "ondblclick") && !has_attr(a, "onmousedown") && !has_attr(a, "onmousemove") && !has_attr(a, "onmouseout") && !has_attr(a, "onmouseover") && !has_attr(a, "onmouseup")) return 0;
 	create_js_event_spec(spec);
@@ -692,16 +612,16 @@ int get_js_events_x(struct js_event_spec **spec, unsigned char *a)
 	return 1;
 }
 
-int get_js_events(unsigned char *a)
+static int get_js_events(unsigned char *a)
 {
 	return get_js_events_x(&format.js_event, a);
 }
 #else
-int get_js_events_x(struct js_event_spec **spec, unsigned char *a)
+static int get_js_events_x(struct js_event_spec **spec, unsigned char *a)
 {
 	return 0;
 }
-int get_js_events(unsigned char *a)
+static int get_js_events(unsigned char *a)
 {
 	return 0;
 }
@@ -753,7 +673,7 @@ void put_chrs(unsigned char *start, int len, int (*put_chars)(void *, unsigned c
 	line_breax = 0;
 }
 
-void kill_until(int ls, ...)
+static void kill_until(int ls, ...)
 {
 	int l;
 	struct html_element *e = &html_top;
@@ -763,7 +683,7 @@ void kill_until(int ls, ...)
 		va_list arg;
 		va_start(arg, ls);
 		while (1) {
-			char *s = va_arg(arg, char *);
+			unsigned char *s = va_arg(arg, unsigned char *);
 			if (!s) break;
 			if (!*s) sk++;
 			else if ((size_t)e->namelen == strlen(s) && !casecmp(e->name, s, strlen(s))) {
@@ -799,7 +719,7 @@ void kill_until(int ls, ...)
 
 int get_num(unsigned char *a, unsigned char *n)
 {
-	char *al;
+	unsigned char *al;
 	if ((al = get_attr_val(a, n))) {
 		char *end;
 		unsigned long s = strtoul(al, &end, 10);
@@ -811,7 +731,7 @@ int get_num(unsigned char *a, unsigned char *n)
 }
 
 /* trunc somehow clips the maximum values. Use 0 to disable truncastion. */
-int parse_width(unsigned char *w, int trunc)
+static int parse_width(unsigned char *w, int trunc)
 {
 	unsigned char *end;
 	int p = 0;
@@ -873,7 +793,7 @@ static inline void set_link_attr(void)
 	memcpy(!(format.attr & AT_INVERT) ? &format.fg : &format.bg, &format.clink, sizeof(struct rgb));
 }
 
-void put_link_line(unsigned char *prefix, unsigned char *linkname, unsigned char *link, unsigned char *target)
+static void put_link_line(unsigned char *prefix, unsigned char *linkname, unsigned char *link, unsigned char *target)
 {
 	html_stack_dup();
 	ln_break(1, line_break_f, ff);
@@ -890,33 +810,33 @@ void put_link_line(unsigned char *prefix, unsigned char *linkname, unsigned char
 	kill_html_stack_item(&html_top);
 }
 
-void html_span(unsigned char *a) { }
+static void html_span(unsigned char *a) { }
 
-void html_bold(unsigned char *a)
+static void html_bold(unsigned char *a)
 { 
 	get_js_events(a); 
 	format.attr |= AT_BOLD; 
 }
 
-void html_italic(unsigned char *a)
+static void html_italic(unsigned char *a)
 {
 	get_js_events(a);
 	format.attr |= AT_ITALIC;
 }
 
-void html_underline(unsigned char *a)
+static void html_underline(unsigned char *a)
 {
 	get_js_events(a); 
 	format.attr |= AT_UNDERLINE;
 }
 
-void html_fixed(unsigned char *a)
+static void html_fixed(unsigned char *a)
 {
 	get_js_events(a);
 	format.attr |= AT_FIXED;
 }
 
-void html_invert(unsigned char *a)
+static void html_invert(unsigned char *a)
 {
 	struct rgb rgb;
 	get_js_events(a);
@@ -926,14 +846,14 @@ void html_invert(unsigned char *a)
 	format.attr ^= AT_INVERT;
 }
 
-void html_a(unsigned char *a)
+static void html_a(unsigned char *a)
 {
-	char *al;
+	unsigned char *al;
 
 	int ev = get_js_events(a);
 
 	if ((al = get_url_val(a, "href"))) {
-		char *all = al;
+		unsigned char *all = al;
 		while (all[0] == ' ') all++;
 		while (all[0] && all[strlen(all) - 1] == ' ') all[strlen(all) - 1] = 0;
 		if (format.link) mem_free(format.link);
@@ -955,7 +875,7 @@ void html_a(unsigned char *a)
 	}
 }
 
-void html_a_special(unsigned char *a, unsigned char *next, unsigned char *eof)
+static void html_a_special(unsigned char *a, unsigned char *next, unsigned char *eof)
 {
 	unsigned char *t;
 	while (next < eof && WHITECHAR(*next)) next++;
@@ -968,14 +888,14 @@ void html_a_special(unsigned char *a, unsigned char *next, unsigned char *eof)
 	mem_free(t);
 }
 
-void html_sub(unsigned char *a)
+static void html_sub(unsigned char *a)
 {
 	get_js_events(a);
 	format.fontsize = 1;
 	format.baseline = -1;
 }
 
-void html_sup(unsigned char *a)
+static void html_sup(unsigned char *a)
 {
 	get_js_events(a);
 	format.fontsize = 1;
@@ -983,13 +903,13 @@ void html_sup(unsigned char *a)
 }
 
 
-void html_font(unsigned char *a)
+static void html_font(unsigned char *a)
 {
-	char *al;
+	unsigned char *al;
 	if ((al = get_attr_val(a, "size"))) {
 		int p = 0;
 		unsigned long s;
-		char *nn = al;
+		unsigned char *nn = al;
 		char *end;
 		if (*al == '+') p = 1, nn++;
 		if (*al == '-') p = -1, nn++;
@@ -1006,7 +926,7 @@ void html_font(unsigned char *a)
 	get_color(a, "color", &format.fg);
 }
 
-void html_img(unsigned char *a)
+static void html_img(unsigned char *a)
 {
 	unsigned char *al;
 	unsigned char *s;
@@ -1156,7 +1076,7 @@ void html_img(unsigned char *a)
 	if (orig_link) mem_free(orig_link);
 }
 
-void html_obj(unsigned char *a, int obj)
+static void html_obj(unsigned char *a, int obj)
 {
 	unsigned char *old_base = format.href_base;
 	unsigned char *url;
@@ -1181,17 +1101,17 @@ void html_obj(unsigned char *a, int obj)
 	if (type) mem_free(type);
 }
 
-void html_embed(unsigned char *a)
+static void html_embed(unsigned char *a)
 {
 	html_obj(a, 0);
 }
 
-void html_object(unsigned char *a)
+static void html_object(unsigned char *a)
 {
 	html_obj(a, 1);
 }
 
-void html_body(unsigned char *a)
+static void html_body(unsigned char *a)
 {
 	get_color(a, "text", &format.fg);
 	get_color(a, "link", &format.clink);
@@ -1203,11 +1123,11 @@ void html_body(unsigned char *a)
 	*/
 }
 
-void html_skip(unsigned char *a) { html_top.invisible = html_top.dontkill = 1; }
+static void html_skip(unsigned char *a) { html_top.invisible = html_top.dontkill = 1; }
 
-void html_title(unsigned char *a) { html_top.invisible = html_top.dontkill = 1; }
+static void html_title(unsigned char *a) { html_top.invisible = html_top.dontkill = 1; }
 
-void html_script(unsigned char *a)
+static void html_script(unsigned char *a)
 {
 	unsigned char *s;
 	s = get_attr_val(a, "src");
@@ -1216,20 +1136,20 @@ void html_script(unsigned char *a)
 	html_skip(a);
 }
 
-void html_noscript(unsigned char *a)
+static void html_noscript(unsigned char *a)
 {
 	if (d_opt->js_enable) html_skip(a);
 }
 
-void html_center(unsigned char *a)
+static void html_center(unsigned char *a)
 {
 	par_format.align = AL_CENTER;
 	if (!table_level && !F) par_format.leftmargin = par_format.rightmargin = 0;
 }
 
-void html_linebrk(unsigned char *a)
+static void html_linebrk(unsigned char *a)
 {
-	char *al;
+	unsigned char *al;
 	if ((al = get_attr_val(a, "align"))) {
 		if (!strcasecmp(al, "left")) par_format.align = AL_LEFT;
 		if (!strcasecmp(al, "right")) par_format.align = AL_RIGHT;
@@ -1242,19 +1162,19 @@ void html_linebrk(unsigned char *a)
 	}
 }
 
-void html_br(unsigned char *a)
+static void html_br(unsigned char *a)
 {
 	html_linebrk(a);
 	if (was_br) ln_break(2, line_break_f, ff);
 	was_br = 1;
 }
 
-void html_form(unsigned char *a)
+static void html_form(unsigned char *a)
 {
 	was_br = 1;
 }
 
-void html_p(unsigned char *a)
+static void html_p(unsigned char *a)
 {
 	if (par_format.leftmargin < margin) par_format.leftmargin = margin;
 	if (par_format.rightmargin < margin) par_format.rightmargin = margin;
@@ -1262,19 +1182,19 @@ void html_p(unsigned char *a)
 	html_linebrk(a);
 }
 
-void html_address(unsigned char *a)
+static void html_address(unsigned char *a)
 {
 	par_format.leftmargin += 1;
 	par_format.align = AL_LEFT;
 }
 
-void html_blockquote(unsigned char *a)
+static void html_blockquote(unsigned char *a)
 {
 	par_format.leftmargin += 2;
 	par_format.align = AL_LEFT;
 }
 
-void html_h(int h, char *a)
+static void html_h(int h, char *a)
 {
 #ifdef G
 	if (F) {
@@ -1308,14 +1228,14 @@ void html_h(int h, char *a)
 	}
 }
 
-void html_h1(unsigned char *a) { html_h(1, a); }
-void html_h2(unsigned char *a) { html_h(2, a); }
-void html_h3(unsigned char *a) { html_h(3, a); }
-void html_h4(unsigned char *a) { html_h(4, a); }
-void html_h5(unsigned char *a) { html_h(5, a); }
-void html_h6(unsigned char *a) { html_h(6, a); }
+static void html_h1(unsigned char *a) { html_h(1, a); }
+static void html_h2(unsigned char *a) { html_h(2, a); }
+static void html_h3(unsigned char *a) { html_h(3, a); }
+static void html_h4(unsigned char *a) { html_h(4, a); }
+static void html_h5(unsigned char *a) { html_h(5, a); }
+static void html_h6(unsigned char *a) { html_h(6, a); }
 
-void html_pre(unsigned char *a)
+static void html_pre(unsigned char *a)
 {
 	format.attr |= AT_FIXED;
 	par_format.align = AL_NO;
@@ -1323,7 +1243,7 @@ void html_pre(unsigned char *a)
 	par_format.rightmargin = 0;
 }
 
-void html_hr(unsigned char *a)
+static void html_hr(unsigned char *a)
 {
 	int i/* = par_format.width - 10*/;
 	int q = get_num(a, "size");
@@ -1359,7 +1279,7 @@ void html_hr(unsigned char *a)
 	kill_html_stack_item(&html_top);
 }
 
-void html_table(unsigned char *a)
+static void html_table(unsigned char *a)
 {
 	par_format.leftmargin = margin;
 	par_format.rightmargin = margin;
@@ -1368,12 +1288,12 @@ void html_table(unsigned char *a)
 	format.attr = 0;
 }
 
-void html_tr(unsigned char *a)
+static void html_tr(unsigned char *a)
 {
 	html_linebrk(a);
 }
 
-void html_th(unsigned char *a)
+static void html_th(unsigned char *a)
 {
 	/*html_linebrk(a);*/
 	kill_until(1, "TD", "TH", "", "TR", "TABLE", NULL);
@@ -1381,7 +1301,7 @@ void html_th(unsigned char *a)
 	put_chrs(" ", 1, put_chars_f, ff);
 }
 
-void html_td(unsigned char *a)
+static void html_td(unsigned char *a)
 {
 	/*html_linebrk(a);*/
 	kill_until(1, "TD", "TH", "", "TR", "TABLE", NULL);
@@ -1389,9 +1309,9 @@ void html_td(unsigned char *a)
 	put_chrs(" ", 1, put_chars_f, ff);
 }
 
-void html_base(unsigned char *a)
+static void html_base(unsigned char *a)
 {
-	char *al;
+	unsigned char *al;
 	if ((al = get_url_val(a, "href"))) {
 		if (format.href_base) mem_free(format.href_base);
 		format.href_base = join_urls(((struct html_element *)html_stack.prev)->attr.href_base, al);
@@ -1404,9 +1324,9 @@ void html_base(unsigned char *a)
 	}
 }
 
-void html_ul(unsigned char *a)
+static void html_ul(unsigned char *a)
 {
-	char *al;
+	unsigned char *al;
 	/*debug_stack();*/
 	par_format.list_level++;
 	par_format.list_number = 0;
@@ -1422,9 +1342,9 @@ void html_ul(unsigned char *a)
 	html_top.dontkill = 1;
 }
 
-void html_ol(unsigned char *a)
+static void html_ol(unsigned char *a)
 {
-	char *al;
+	unsigned char *al;
 	int st;
 	par_format.list_level++;
 	st = get_num(a, "start");
@@ -1447,11 +1367,11 @@ void html_ol(unsigned char *a)
 	html_top.dontkill = 1;
 }
 
-void html_li(unsigned char *a)
+static void html_li(unsigned char *a)
 {
 	/*kill_until(0, "", "UL", "OL", NULL);*/
 	if (!par_format.list_number) {
-		char x[7] = "*&nbsp;";
+		unsigned char x[7] = "*&nbsp;";
 		if ((par_format.flags & P_LISTMASK) == P_O) x[0] = 'o';
 		if ((par_format.flags & P_LISTMASK) == P_PLUS) x[0] = '+';
 		if (F) par_format.leftmargin += 2;
@@ -1460,8 +1380,8 @@ void html_li(unsigned char *a)
 		par_format.align = AL_LEFT;
 		putsp = -1;
 	} else {
-		char c = 0;
-		char n[32];
+		unsigned char c = 0;
+		unsigned char n[32];
 		int t = par_format.flags & P_LISTMASK;
 		int s = get_num(a, "value");
 		if (F) par_format.leftmargin += 4;
@@ -1473,7 +1393,7 @@ void html_li(unsigned char *a)
 		} else if (t == P_ROMAN || t == P_roman) {
 			roman(n, par_format.list_number);
 			if (t == P_ROMAN) {
-				char *x;
+				unsigned char *x;
 				for (x = n; *x; x++) *x = upcase(*x);
 			}
 		} else sprintf(n, "%d", par_format.list_number);
@@ -1481,14 +1401,14 @@ void html_li(unsigned char *a)
 		put_chrs(".&nbsp;", 7, put_chars_f, ff);
 		if (!F) par_format.leftmargin += strlen(n) + c + 2;
 		par_format.align = AL_LEFT;
+		html_top.next->parattr.list_number = par_format.list_number + 1;
 		par_format.list_number = 0;
-		html_top.next->parattr.list_number++;
 		putsp = -1;
 	}
 	line_breax = 2;
 }
 
-void html_dl(unsigned char *a)
+static void html_dl(unsigned char *a)
 {
 	par_format.flags &= ~P_COMPACT;
 	if (has_attr(a, "compact")) par_format.flags |= P_COMPACT;
@@ -1504,7 +1424,7 @@ void html_dl(unsigned char *a)
 	}
 }
 
-void html_dt(unsigned char *a)
+static void html_dt(unsigned char *a)
 {
 	kill_until(0, "", "DL", NULL);
 	par_format.align = AL_LEFT;
@@ -1513,7 +1433,7 @@ void html_dt(unsigned char *a)
 		ln_break(2, line_break_f, ff);
 }
 
-void html_dd(unsigned char *a)
+static void html_dd(unsigned char *a)
 {
 	kill_until(0, "", "DL", NULL);
 	if ((par_format.leftmargin = par_format.dd_margin + (table_level ? 3 : 8)) > par_format.width * 2 / 3 && !table_level)
@@ -1521,14 +1441,14 @@ void html_dd(unsigned char *a)
 	par_format.align = AL_LEFT;
 }
 
-void get_html_form(unsigned char *a, struct form *form)
+static void get_html_form(unsigned char *a, struct form *form)
 {
 	unsigned char *al;
 	unsigned char *ch;
 	form->method = FM_GET;
 	if ((al = get_attr_val(a, "method"))) {
 		if (!strcasecmp(al, "post")) {
-			char *ax;
+			unsigned char *ax;
 			form->method = FM_POST;
 			if ((ax = get_attr_val(a, "enctype"))) {
 				if (!strcasecmp(ax, "multipart/form-data"))
@@ -1539,7 +1459,7 @@ void get_html_form(unsigned char *a, struct form *form)
 		mem_free(al);
 	}
 	if ((al = get_url_val(a, "action"))) {
-		char *all = al;
+		unsigned char *all = al;
 		while (all[0] == ' ') all++;
 		while (all[0] && all[strlen(all) - 1] == ' ') all[strlen(all) - 1] = 0;
 		form->action = join_urls(format.href_base, all);
@@ -1564,7 +1484,7 @@ void get_html_form(unsigned char *a, struct form *form)
 	form->num = a - startf;
 }
 
-void find_form_for_input(unsigned char *i)
+static void find_form_for_input(unsigned char *i)
 {
 	unsigned char *s, *ss, *name, *attr, *lf, *la;
 	int namelen;
@@ -1578,9 +1498,15 @@ void find_form_for_input(unsigned char *i)
 		get_html_form(last_form_attr, &form);
 		return;
 	}
-	if (last_form_tag && last_input_tag && i > last_input_tag) s = last_form_tag;
-	else s = startf;
-	lf = NULL, la = NULL;
+	if (last_form_tag && last_input_tag && i > last_input_tag) {
+		if (parse_element(last_form_tag, i, &name, &namelen, &la, &s))
+			internal("couldn't parse already parsed tag");
+		lf = last_form_tag;
+		s = last_input_tag;
+	} else {
+		lf = NULL, la = NULL;
+		s = startf;
+	}
 	se:
 	while (s < i && *s != '<') sp:s++;
 	if (s >= i) goto end_parse;
@@ -1601,12 +1527,14 @@ void find_form_for_input(unsigned char *i)
 		last_form_attr = la;
 		last_input_tag = i;
 		get_html_form(la, &form);
-	} else last_form_tag = NULL;
+	} else {
+		last_form_tag = NULL;
+	}
 }
 
-void html_button(unsigned char *a)
+static void html_button(unsigned char *a)
 {
-	char *al;
+	unsigned char *al;
 	struct form_control *fc;
 	find_form_for_input(a);
 	fc = mem_calloc(sizeof(struct form_control));
@@ -1649,7 +1577,7 @@ void html_button(unsigned char *a)
 	put_chrs(" ", 1, put_chars_f, ff);*/
 }
 
-void html_input(unsigned char *a)
+static void html_input(unsigned char *a)
 {
 	int i;
 	unsigned char *al;
@@ -1686,7 +1614,7 @@ void html_input(unsigned char *a)
 	if (fc->type == FC_TEXT || fc->type == FC_PASSWORD) fc->default_value = get_attr_val(a, "value");
 	else if (fc->type != FC_FILE) fc->default_value = get_exact_attr_val(a, "value");
 	if (fc->type == FC_CHECKBOX && !fc->default_value) fc->default_value = stracpy("on");
-	if ((fc->size = get_num(a, "size")) == -1) fc->size = HTML_DEFAULT_INPUT_SIZE;
+	if ((fc->size = get_num(a, "size")) <= 0) fc->size = HTML_DEFAULT_INPUT_SIZE;
 	fc->size++;
 	if (fc->size > d_opt->xw) fc->size = d_opt->xw;
 	if ((fc->maxlength = get_num(a, "maxlength")) == -1) fc->maxlength = MAXINT / 4;
@@ -1763,9 +1691,9 @@ void html_input(unsigned char *a)
 	special_f(ff, SP_CONTROL, fc);
 }
 
-void html_select(unsigned char *a)
+static void html_select(unsigned char *a)
 {
-	char *al;
+	unsigned char *al;
 	if (!(al = get_attr_val(a, "name"))) return;
 	html_top.dontkill = 1;
 	if (format.select) mem_free(format.select);
@@ -1773,7 +1701,7 @@ void html_select(unsigned char *a)
 	format.select_disabled = 2 * has_attr(a, "disabled");
 }
 
-void html_option(unsigned char *a)
+static void html_option(unsigned char *a)
 {
 	struct form_control *fc;
 	unsigned char *val;
@@ -1861,7 +1789,7 @@ void clr_spaces(unsigned char *name)
 int menu_stack_size;
 struct menu_item **menu_stack;
 
-void new_menu_item(unsigned char *name, long data, int fullname)
+static void new_menu_item(unsigned char *name, long data, int fullname)
 	/* name == NULL - up;	data == -1 - down */
 {
 	struct menu_item *top, *item, *nmenu = NULL; /* no uninitialized warnings */
@@ -1905,7 +1833,7 @@ void new_menu_item(unsigned char *name, long data, int fullname)
 	if (!name) menu_stack_size--;
 }
 
-void init_menu(void)
+static void init_menu(void)
 {
 	menu_stack_size = 0;
 	menu_stack = DUMMY;
@@ -1922,7 +1850,7 @@ void free_menu(struct menu_item *m) /* Grrr. Recursion */
 	mem_free(m);
 }
 
-struct menu_item *detach_menu(void)
+static struct menu_item *detach_menu(void)
 {
 	struct menu_item *i = NULL;
 	if (menu_stack_size) i = menu_stack[0];
@@ -1930,13 +1858,13 @@ struct menu_item *detach_menu(void)
 	return i;
 }
 
-void destroy_menu(void)
+static void destroy_menu(void)
 {
 	if (menu_stack && menu_stack != DUMMY) free_menu(menu_stack[0]);
 	detach_menu();
 }
 
-void menu_labels(struct menu_item *m, unsigned char *base, unsigned char **lbls)
+static void menu_labels(struct menu_item *m, unsigned char *base, unsigned char **lbls)
 {
 	unsigned char *bs;
 	for (; m->text; m++) {
@@ -1954,7 +1882,7 @@ void menu_labels(struct menu_item *m, unsigned char *base, unsigned char **lbls)
 	}
 }
 
-int menu_contains(struct menu_item *m, int f)
+static int menu_contains(struct menu_item *m, int f)
 {
 	if (m->func != MENU_FUNC do_select_submenu) return (my_intptr_t)m->data == f;
 	for (m = m->data; m->text; m++) if (menu_contains(m, f)) return 1;
@@ -1973,7 +1901,7 @@ void do_select_submenu(struct terminal *term, struct menu_item *menu, struct ses
 	do_menu_selected(term, menu, ses, sel);
 }
 
-int do_html_select(unsigned char *attr, unsigned char *html, unsigned char *eof, unsigned char **end, void *f)
+static int do_html_select(unsigned char *attr, unsigned char *html, unsigned char *eof, unsigned char **end, void *f)
 {
 	struct form_control *fc;
 	unsigned char *t_name, *t_attr, *en;
@@ -1987,7 +1915,7 @@ int do_html_select(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	unsigned char **val, **lbls;
 	int order, preselect, group;
 	int i, mw;
-	if (has_attr(attr, "multiple")) return 1;
+	if (has_attr(attr, "multiple") || dmp) return 1;
 	find_form_for_input(attr);
 	lbl = NULL;
 	lbl_l = 0;
@@ -2086,7 +2014,7 @@ int do_html_select(unsigned char *attr, unsigned char *html, unsigned char *eof,
 		if (group) new_menu_item(NULL, -1, 0), group = 0;
 	}
 	if (t_namelen == 8 && !casecmp(t_name, "OPTGROUP", 8)) {
-		char *la;
+		unsigned char *la;
 		if (!(la = get_attr_val(t_attr, "label"))) la = stracpy("");
 		new_menu_item(convert_string(ct, la, strlen(la), d_opt), -1, 0);
 		mem_free(la);
@@ -2126,7 +2054,7 @@ int do_html_select(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	format.attr |= AT_BOLD | AT_FIXED;
 	format.fontsize = 3;
 	mw = 0;
-	for (i = 0; i < order; i++) if (lbls[i] && utf8len(lbls[i]) > mw) mw = utf8len(lbls[i]);
+	for (i = 0; i < order; i++) if (lbls[i] && cp_len(d_opt->cp, lbls[i]) > mw) mw = cp_len(d_opt->cp, lbls[i]);
 	for (i = 0; i < mw; i++) put_chrs("_", 1, put_chars_f, f);
 	kill_html_stack_item(&html_top);
 	put_chrs("]", 1, put_chars_f, f);
@@ -2135,12 +2063,12 @@ int do_html_select(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	return 0;
 }
 
-void html_textarea(unsigned char *a)
+static void html_textarea(unsigned char *a)
 {
 	internal("This should be never called");
 }
 
-void do_html_textarea(unsigned char *attr, unsigned char *html, unsigned char *eof, unsigned char **end, void *f)
+static void do_html_textarea(unsigned char *attr, unsigned char *html, unsigned char *eof, unsigned char **end, void *f)
 {
 	struct form_control *fc;
 	unsigned char *p, *t_name, *w;
@@ -2215,23 +2143,25 @@ void do_html_textarea(unsigned char *attr, unsigned char *html, unsigned char *e
 	special_f(f, SP_CONTROL, fc);
 }
 
-void html_iframe(unsigned char *a)
+static void html_iframe(unsigned char *a)
 {
 	unsigned char *name, *url;
 	if (!(url = get_url_val(a, "src"))) return;
+	if (!*url) goto free_url_ret;
 	if (!(name = get_attr_val(a, "name"))) name = stracpy("");
 	if (*name) put_link_line("IFrame: ", name, url, d_opt->framename);
 	else put_link_line("", "IFrame", url, d_opt->framename);
 	mem_free(name);
+	free_url_ret:
 	mem_free(url);
 }
 
-void html_noframes(unsigned char *a)
+static void html_noframes(unsigned char *a)
 {
 	if (d_opt->frames) html_skip(a);
 }
 
-void html_frame(unsigned char *a)
+static void html_frame(unsigned char *a)
 {
 	unsigned char *name, *u2, *url;
 	if (!(u2 = get_url_val(a, "src"))) {
@@ -2251,7 +2181,7 @@ void html_frame(unsigned char *a)
 	if (!d_opt->frames || !html_top.frameset) put_link_line("Frame: ", name, url, "");
 	else {
 		struct frame_param fp;
-		char *scroll = get_attr_val(a, "scrolling");
+		unsigned char *scroll = get_attr_val(a, "scrolling");
 		fp.name = name;
 		fp.url = url;
 		fp.parent = html_top.frameset;
@@ -2271,7 +2201,7 @@ void html_frame(unsigned char *a)
 	mem_free(url);
 }
 
-void parse_frame_widths(unsigned char *a, int ww, int www, int **op, int *olp)
+static void parse_frame_widths(unsigned char *a, int ww, int www, int **op, int *olp)
 {
 	unsigned char *aa;
 	int q, qq, i, d, nn;
@@ -2359,7 +2289,7 @@ void parse_frame_widths(unsigned char *a, int ww, int www, int **op, int *olp)
 	}
 }
 
-void html_frameset(unsigned char *a)
+static void html_frameset(unsigned char *a)
 {
 	int x, y;
 	struct frameset_param fp;
@@ -2393,7 +2323,7 @@ void html_frameset(unsigned char *a)
 	mem_free(d);
 }
 
-/*void html_frameset(unsigned char *a)
+/*static void html_frameset(unsigned char *a)
 {
 	int w;
 	int horiz = 0;
@@ -2431,7 +2361,7 @@ void html_frameset(unsigned char *a)
 	mem_free(c);
 }*/
 
-void html_link(unsigned char *a)
+static void html_link(unsigned char *a)
 {
 	unsigned char *name, *url, *title;
 	if ((name = get_attr_val(a, "type"))) {
@@ -2457,6 +2387,13 @@ void html_link(unsigned char *a)
 	    !strcasecmp(name, "File-List") ||
 	    !strcasecmp(name, "Edit-Time-Data") ||
 	    !casecmp(name, "schema", 6)) goto skip;
+	if (!strcasecmp(name, "prefetch") ||
+	    !strcasecmp(name, "dns-prefetch")) {
+		unsigned char *pre_url = join_urls(format.href_base, url);
+		if (!dmp) load_url(pre_url, format.href_base, NULL, PRI_PRELOAD, NC_ALWAYS_CACHE);
+		mem_free(pre_url);
+		goto skip;
+	}
 	if ((title = get_attr_val(a, "title"))) {
 		add_to_strn(&name, ": ");
 		add_to_strn(&name, title);
@@ -2469,7 +2406,7 @@ void html_link(unsigned char *a)
 }
 
 struct element_info {
-	char *name;
+	unsigned char *name;
 	void (*func)(unsigned char *);
 	int linebreak;
 	int nopair; /* Somehow relates to paired elements */
@@ -2578,19 +2515,19 @@ unsigned char *skip_comment(unsigned char *html, unsigned char *eof)
 	return eof;
 }
 
-void process_head(unsigned char *head)
+static void process_head(unsigned char *head)
 {
 	unsigned char *r, *p;
 	struct refresh_param rp;
 	if ((r = parse_http_header(head, "Refresh", NULL))) {
 		if (!d_opt->auto_refresh) {
-			if ((p = parse_header_param(r, "URL")) || (p = parse_header_param(r, ""))) {
+			if ((p = parse_header_param(r, "URL", 0)) || (p = parse_header_param(r, "", 0))) {
 				put_link_line("Refresh: ", p, p, d_opt->framename);
 				mem_free(p);
 			}
 		} else {
-			rp.url = parse_header_param(r, "URL");
-			if (!rp.url) rp.url = parse_header_param(r, "");
+			rp.url = parse_header_param(r, "URL", 0);
+			if (!rp.url) rp.url = parse_header_param(r, "", 0);
 			rp.time = atoi(r);
 			if (rp.time < 1) rp.time = 1;
 			special_f(ff, SP_REFRESH, &rp);
@@ -2600,7 +2537,7 @@ void process_head(unsigned char *head)
 	}
 }
 
-int qd(unsigned char *html, unsigned char *eof, int *len)
+static int qd(unsigned char *html, unsigned char *eof, int *len)
 {
 	int l;
 	*len = 1;
@@ -2647,6 +2584,7 @@ void parse_html(unsigned char *html, unsigned char *eof, int (*put_chars)(void *
 	lt = html;
 	while (html < eof) {
 		unsigned char *name, *attr, *end;
+		unsigned char *a;
 		int namelen;
 		struct element_info *ei;
 		int inv;
@@ -2746,16 +2684,43 @@ void parse_html(unsigned char *html, unsigned char *eof, int (*put_chars)(void *
 		}
 		html = end;
 		for (ei = elements; ei->name; ei++) {
-			if (ei->name &&
-			   (strlen(ei->name) != (size_t)namelen || casecmp(ei->name, name, namelen)))
+			if (strlen(ei->name) != (size_t)namelen || casecmp(ei->name, name, namelen))
 				continue;
 			if (!inv) {
-				char *a;
+				int display_none = 0;
+				int noskip = 0;
 				ln_break(ei->linebreak, line_break, f);
-				if (ei->name && ((a = get_attr_val(attr, "id")))) {
+				if ((a = get_attr_val(attr, "id"))) {
 					special(f, SP_TAG, a);
 					mem_free(a);
 				}
+				if ((a = get_attr_val(attr, "style"))) {
+					unsigned char *d, *s;
+
+					if (!strcasecmp(ei->name, "INPUT")) {
+						unsigned char *aa = get_attr_val(attr, "type");
+						if (aa) {
+							if (!strcasecmp(aa, "hidden"))
+								noskip = 1;
+							mem_free(aa);
+						}
+					}
+
+					for (d = s = a; *s; s++) if (*s > ' ') *d++ = *s;
+					*d = 0;
+					display_none |= !casecmp(a, "display:none", 12) && !noskip;
+					mem_free(a);
+				}
+				if (display_none) {
+					if (ei->nopair == 1) goto set_lt;
+					html_stack_dup();
+					html_top.name = name;
+					html_top.namelen = namelen;
+					html_top.options = attr;
+					html_top.linebreak = 0;
+					html_top.invisible = 1;
+					goto set_lt;
+				}
 				if (!html_top.invisible) {
 					int a = par_format.align == AL_NO;
 					struct par_attrib pa = par_format;
@@ -2809,7 +2774,7 @@ void parse_html(unsigned char *html, unsigned char *eof, int (*put_chars)(void *
 				if (ei->nopair == 1 || ei->nopair == 3) break;
 				/*debug_stack();*/
 				foreach(e, html_stack) {
-					if (e->linebreak && !ei->linebreak && ei->name) xxx = 1;
+					if (e->linebreak && !ei->linebreak) xxx = 1;
 					if (e->namelen != namelen || casecmp(e->name, name, e->namelen)) {
 						if (e->dontkill) break;
 						else continue;
@@ -2830,7 +2795,13 @@ void parse_html(unsigned char *html, unsigned char *eof, int (*put_chars)(void *
 			}
 			goto set_lt;
 		}
-		goto set_lt; /* unreachable */
+		if (!inv) {
+			if ((a = get_attr_val(attr, "id"))) {
+				special(f, SP_TAG, a);
+				mem_free(a);
+			}
+		}
+		goto set_lt;
 	}
 	put_chrs(lt, html - lt, put_chars, f);
 	ln_break(1, line_break, f);
@@ -2840,7 +2811,7 @@ void parse_html(unsigned char *html, unsigned char *eof, int (*put_chars)(void *
 	was_br = 0;
 }
 
-void scan_area_tag(unsigned char *attr, unsigned char *name, unsigned char **ptr, struct memory_list **ml)
+static void scan_area_tag(unsigned char *attr, unsigned char *name, unsigned char **ptr, struct memory_list **ml)
 {
 	unsigned char *v;
 	if ((v = get_attr_val(attr, name))) {
@@ -3034,10 +3005,10 @@ void scan_http_equiv(unsigned char *s, unsigned char *eof, unsigned char **head,
 		goto se;
 	}
 	if (namelen == 4 && !casecmp(name, "BODY", 4)) {
-		if (background) *background = get_attr_val(attr, "background");
-		if (bgcolor) *bgcolor = get_attr_val(attr, "bgcolor");
+		if (background) *background = get_attr_val(attr, "background"), background = NULL;
+		if (bgcolor) *bgcolor = get_attr_val(attr, "bgcolor"), bgcolor = NULL;
 		if (j) get_js_events_x(j, attr);
-		return;
+		/*return;*/
 	}
 	if (title && !tlen && namelen == 5 && !casecmp(name, "TITLE", 5)) {
 		unsigned char *s1;
