@@ -28,7 +28,7 @@
 #include <vgamouse.h>
 #include "arrow.inc"
 
-extern struct itrm *ditrm;
+struct itrm *svgalib_kbd;
 
 extern struct graphics_driver svga_driver;
 static int mouse_x, mouse_y, mouse_buttons; /* For tracking the state of the mouse */
@@ -91,25 +91,25 @@ struct modeline{
 #define END_MOUSE if (mouse_hidden) show_mouse();
 
 /* Actual vga mode definition */
-int vga_linewidth;		/* Prepared out from vga_getmodeinfo */
-int xsize, ysize;		/* Prepared out from vga_getmodeinfo */
-int vga_bytes;			/* Prepared out from vga_getmodeinfo */
-int vga_colors;			/* Prepared out from vga_getmodeinfo */
-int vga_misordered;		/* Prepared out from vga_getmodeinfo */
-int vga_linear;			/* 1 linear mode, 0 nonlinear mode (paged) */
-int palette_depth;		/* 6 for normal VGA, 8 for VGA which supports 8 bit DAC */
-int accel_avail;		/* Which accel fns are available */
-int do_sync;			/* Tells the "normal" memory operations (those
+static int vga_linewidth;		/* Prepared out from vga_getmodeinfo */
+static int xsize, ysize;		/* Prepared out from vga_getmodeinfo */
+static int vga_bytes;			/* Prepared out from vga_getmodeinfo */
+static int vga_colors;			/* Prepared out from vga_getmodeinfo */
+static int vga_misordered;		/* Prepared out from vga_getmodeinfo */
+static int vga_linear;			/* 1 linear mode, 0 nonlinear mode (paged) */
+static int palette_depth;		/* 6 for normal VGA, 8 for VGA which supports 8 bit DAC */
+static int accel_avail;		/* Which accel fns are available */
+static int do_sync;			/* Tells the "normal" memory operations (those
 				 * that do not use accelerator) to do
 				 * vga_accel(ACCEL_SYNC) before writing into the
 				 * memory.
 				 */
-int vga_page=-1;
-int mode_x;			/* 1 if mode_X organization is set up */
-int bmpixelsize;		/* Number of bytes per pixel in bitmap */
-unsigned char *my_graph_mem;
-unsigned char *scroll_buffer = NULL; /* For paged scrolling only */
-struct modeline modes[]={
+static int vga_page=-1;
+static int mode_x;			/* 1 if mode_X organization is set up */
+static int bmpixelsize;		/* Number of bytes per pixel in bitmap */
+static unsigned char *my_graph_mem;
+static unsigned char *scroll_buffer = NULL; /* For paged scrolling only */
+static struct modeline modes[]={
 #ifdef G320x200x16
 	{"320x200x16", G320x200x16 },
 #endif
@@ -658,8 +658,12 @@ static void svga_shutdown_driver(void)
 #ifndef __SPAD__
 	kill_timer(svgalib_timer_id);
 #endif
-	vga_setmode(TEXT);
-	svgalib_free_trm(ditrm);
+	if (vga_setmode(TEXT) < 0) {
+		error("ERROR: vga_setmode failed");
+		fatal_tty_exit();
+		exit(RET_FATAL);
+	}
+	svgalib_free_trm(svgalib_kbd);
 	if (svga_driver_param)mem_free(svga_driver_param);
 	install_signal_handler(SIGINT, NULL, NULL, 0);
 }
@@ -949,7 +953,7 @@ static void fill_area_linear(struct graphics_device *dev, int left, int top, int
 	SYNC
 	dest=my_graph_mem+top*vga_linewidth+left*vga_bytes;
 	for (y=bottom-top;y;y--){
-		pixel_set(dest,(right-left)*vga_bytes,&color);
+		pixel_set(dest,(right-left)*vga_bytes,do_not_optimize_here(&color));
 		dest+=vga_linewidth;
 	}
 	END_MOUSE
@@ -1072,7 +1076,7 @@ static void draw_hline_linear(struct graphics_device *dev, int left, int y, int
 	HLINE_CLIP_PREFACE
 	SYNC	
 	dest=my_graph_mem+y*vga_linewidth+left*vga_bytes;
-	pixel_set(dest,(right-left)*vga_bytes,&color);
+	pixel_set(dest,(right-left)*vga_bytes,do_not_optimize_here(&color));
 	END_MOUSE
 }
 
@@ -1142,7 +1146,7 @@ static void draw_vline_paged_1(struct graphics_device *dev, int x, int top, int
 static void draw_vline_paged_2(struct graphics_device *dev, int x, int top, int bottom, long color)
 {
 	int dest,page,n,paga,remains;
-	int word=*(t2c *)(void *)&color;
+	int word=*(t2c *)do_not_optimize_here((void *)&color);
 	VLINE_CLIP_PREFACE;
 	SYNC
 	dest=top*vga_linewidth+(x<<1);
@@ -1178,7 +1182,7 @@ static void draw_vline_paged_2(struct graphics_device *dev, int x, int top, int
 static void draw_vline_paged_4(struct graphics_device *dev, int x, int top, int bottom, long color)
 {
 	unsigned long dest,page,paga,remains,n;
-	t4c val=*(t4c *)(void *)&color;
+	t4c val=*(t4c *)do_not_optimize_here((void *)&color);
 
 	VLINE_CLIP_PREFACE;
 	SYNC
@@ -1311,7 +1315,6 @@ static int hscroll_accel(struct graphics_device *dev, struct rect_set **ignore,
 {
 	HSCROLL_CLIP_PREFACE
 
-	ignore=NULL;
 	if (sc>0){
 		/* Move data to the right */
 		vga_accel(ACCEL_SCREENCOPY,dev->clip.x1,dev->clip.y1,dev->clip.x1+sc,dev->clip.y1
@@ -1337,7 +1340,6 @@ static int vscroll_accel(struct graphics_device *dev, struct rect_set **ignore,
 {
 	VSCROLL_CLIP_PREFACE
 
-	ignore=NULL;
 	if (sc>0){
 		/* Move down */
 		vga_accel(ACCEL_SCREENCOPY,dev->clip.x1,dev->clip.y1,dev->clip.x1,dev->clip.y1+sc
@@ -1357,7 +1359,6 @@ static int hscroll_scansegment(struct graphics_device *dev, struct rect_set **ig
 	int len;
 	HSCROLL_CLIP_PREFACE
 	SYNC	
-	ignore=NULL;
 	if (sc>0){
 		/* Right */
 		len=dev->clip.x2-dev->clip.x1-sc;
@@ -1384,7 +1385,6 @@ static int hscroll_linear(struct graphics_device *dev, struct rect_set **ignore,
 	int len;
 	HSCROLL_CLIP_PREFACE
 	SYNC
-	ignore=NULL;
 	if (sc>0){
 		len=(dev->clip.x2-dev->clip.x1-sc)*vga_bytes;
 		src=my_graph_mem+vga_linewidth*dev->clip.y1+dev->clip.x1*vga_bytes;
@@ -1414,7 +1414,6 @@ static int vscroll_scansegment(struct graphics_device *dev, struct rect_set **ig
 	int len;
 	VSCROLL_CLIP_PREFACE
 	SYNC
-	ignore=NULL;
 	len=dev->clip.x2-dev->clip.x1;
 	if (sc>0){
 		/* Down */
@@ -1440,7 +1439,6 @@ static int vscroll_linear(struct graphics_device *dev, struct rect_set **ignore,
 	int len;
 	VSCROLL_CLIP_PREFACE
 	SYNC
-	ignore=NULL;
 	len=(dev->clip.x2-dev->clip.x1)*vga_bytes;
 	if (sc>0){
 		/* Down */
@@ -1496,7 +1494,6 @@ static int vscroll_paged(struct graphics_device *dev, struct rect_set **ignore,
 	int len;
 	VSCROLL_CLIP_PREFACE
 	SYNC
-	ignore=NULL;
 	len=(dev->clip.x2-dev->clip.x1)*vga_bytes;
 	if (sc>0){
 		/* Down */
@@ -1531,7 +1528,6 @@ static int hscroll_paged(struct graphics_device *dev, struct rect_set **ignore,
 
 	HSCROLL_CLIP_PREFACE
 	SYNC	
-	ignore=NULL;
 	if (sc>0){
 		len=(dev->clip.x2-dev->clip.x1-sc)*vga_bytes;
 		src=vga_linewidth*dev->clip.y1+dev->clip.x1*vga_bytes;
@@ -1834,7 +1830,9 @@ void dump_mode_info_into_file(vga_modeinfo* i)
 
 static void svgalib_key_in(void *p, struct event *ev, int size)
 {
-	if (size != sizeof(struct event) || ev->ev != EV_KBD) return;
+	if (size != sizeof(struct event)) return;
+	if (ev->ev == EV_ABORT) terminate_loop = 1;
+	if (ev->ev != EV_KBD) return;
 	if ((ev->y & KBD_ALT) && ev->x >= '0' && ev->x <= '9') {
 		switch_virtual_device((ev->x - '1' + 10) % 10);
 		return;
@@ -2335,7 +2333,7 @@ static void setup_mode(int mode)
 	if (!vga_linear){
 		vga_page=-1;
 	}
-	vga_misordered=!!i->flags&RGB_MISORDERED;
+	vga_misordered=!!(i->flags&RGB_MISORDERED);
 	/*dump_mode_info_into_file(i);*/
 	svga_driver.depth=0;
 	svga_driver.depth|=vga_misordered<<8;
@@ -2424,8 +2422,10 @@ static unsigned char *svga_init_driver(unsigned char *param, unsigned char *disp
 		if (j) {
 			add_to_str(&m, &l, "Video mode ");
 			add_to_str(&m, &l, param);
-			add_to_str(&m, &l, " not supported by ");
-			add_to_str(&m, &l, j == 2 ? "your video card" : "svgalib");
+			add_to_str(&m, &l,
+				j == 1 ? " not supported by svgalib" :
+				j == 2 ? " not supported by your video card" :
+				" could not be set");
 			add_to_str(&m, &l, ".\n");
 		} else add_to_str(&m, &l, "There is no default video mode.\n");
 		for (j=0;(size_t)j<sizeof(modes)/sizeof(*modes);j++) if (vga_hasmode(modes[j].number)) {
@@ -2462,9 +2462,18 @@ static unsigned char *svga_init_driver(unsigned char *param, unsigned char *disp
 	svgalib_timer_id=install_timer(100,vtswitch_handler,NULL);
 	if (vga_runinbackground_version()>=1) vga_runinbackground(1);
 #endif
-	vga_setmode(vga_mode=modes[j].number);
+	if (vga_setmode(modes[j].number) < 0) {
+#ifndef __SPAD__
+		kill_timer(svgalib_timer_id);
+#endif
+		vga_unlockvc();
+		shutdown_virtual_devices();
+		j = 3;
+		goto not_found;
+	}
+	vga_mode=modes[j].number;
 	setup_mode(modes[j].number);
-	handle_svgalib_keyboard((void (*)(void *, unsigned char *, int))svgalib_key_in);
+	svgalib_kbd = handle_svgalib_keyboard((void (*)(void *, unsigned char *, int))svgalib_key_in);
 
 	if (mouse_works){
 		if ((unsigned)arrow_area > (unsigned)MAXINT / bmpixelsize) overalloc();
@@ -2486,7 +2495,7 @@ static unsigned char *svga_init_driver(unsigned char *param, unsigned char *disp
 	}
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGTSTP, SIG_IGN);
-	install_signal_handler(SIGINT, (void (*)(void *))svga_ctrl_c, ditrm, 0);
+	install_signal_handler(SIGINT, (void (*)(void *))svga_ctrl_c, svgalib_kbd, 0);
 	return NULL;
 }
 
@@ -2503,7 +2512,7 @@ static int svga_get_filled_bitmap(struct bitmap *dest, long color)
 	if ((unsigned)dest->x * (unsigned)dest->y > MAXINT / bmpixelsize) overalloc();
 	n=dest->x*dest->y*bmpixelsize;
 	dest->data=mem_alloc(n);
-	pixel_set(dest->data,n,&color);
+	pixel_set(dest->data,n,do_not_optimize_here(&color));
 	dest->skip=dest->x*bmpixelsize;
 	dest->flags=0;
 	return 0;
@@ -2534,13 +2543,17 @@ static int vga_block(struct graphics_device *dev)
 			backup_virtual_device=current_virtual_device;
 			current_virtual_device=NULL;
 		}
-		svgalib_block_itrm(ditrm);
+		svgalib_block_itrm(svgalib_kbd);
 		if (mouse_works){
 			hide_mouse();
 			/* mouse_close(); This is not necessary as it is
 				handled by vga_setmode(TEXT). */
 		}
-		vga_setmode(TEXT);
+		if (vga_setmode(TEXT) < 0) {
+			error("ERROR: vga_setmode failed");
+			fatal_tty_exit();
+			exit(RET_FATAL);
+		}
 	}
 	flags|=2;
 	return overridden;
@@ -2557,13 +2570,17 @@ static int vga_unblock(struct graphics_device *dev)
 	flags&=~2;
 	if (!flags) current_virtual_device=backup_virtual_device;
 	vga_setmousesupport(1);
-	vga_setmode(vga_mode);
+	if (vga_setmode(vga_mode) < 0) {
+		error("ERROR: vga_setmode failed");
+		fatal_tty_exit();
+		exit(RET_FATAL);
+	}
 	setup_mode(vga_mode);
 	if (mouse_works){
 		show_mouse();
 		mouse_seteventhandler(mouse_event_handler);
 	}
-	svgalib_unblock_itrm(ditrm);
+	svgalib_unblock_itrm(svgalib_kbd);
 	if (current_virtual_device) current_virtual_device->redraw_handler(current_virtual_device
 			,&current_virtual_device->size);
 	return 0;
